@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
@@ -7,6 +7,12 @@
  *
  * Revisions: 
  *
+ *  01/05/01  [rcg]
+ *            - Added Qstrlen, as a safe strlen.
+ *
+ *  11/14/00  [rcg]
+ *            - Recognize '%p'
+ *
  *  06/20/00  [rcg]
  *            - width and limit now size_t; separate bWidth and bLimit
  *              used to know if width or limit is in use.
@@ -83,28 +89,26 @@
  * Copy chars and update pointer and remaining space count.
  *
  * Parameters:
- *    s:   pointer to char * for destination.
- *    m:   pointer to source string.
- *    l:   maximum chars to copy.
- *    n:   pointer to remaining space count.
+ *    ppDest: pointer to char * for destination.
+ *    pSrc:   pointer to source string.
+ *    len:    maximum chars to copy.
+ *    left:   pointer to remaining space count.
  *
  * Results:
- *    String 'm' is appended to 's'; 's' is updated to
- *    point at terminating null; 'n' is decremented by
+ *    String 'pSrc' is appended to *'ppDest'; 'ppDest' is updated to
+ *    point at terminating null; 'left' is decremented by
  *    number of characters copied.
  *
  *    Return value is number of characters copied.
  */
 static int 
-copy_buf ( char **s, char *m, size_t l, size_t *n )
+copy_buf ( char **ppDest, char *pSrc, size_t len, size_t *left )
 {
-    l = MIN ( strlen(m), l );
-    strncpy ( *s, m, l );
-    
-    l = MIN ( l, strlen(m) );
-    *n     -= l;
-    *s     += l;
-    return    l;
+    len = Qstrlen ( pSrc, len );
+    strncpy ( *ppDest, pSrc, len );    
+    *left   -= len;
+    *ppDest += len;
+    return     len;
 }
 
 
@@ -215,6 +219,25 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                 nState = IN_FORM;
                 break;
 
+            case 'p':
+                /*
+                 * Ensure width or max length, if specified, does not exceed
+                 * available space.
+                 */
+                if ( bWidth || bLimit )
+                    if ( MAX ( width,   limit ) >
+                         MIN ( BUFSIZE, nSize )
+                       )
+                        return -1;
+
+                *f++ = *p;
+                *f   = '\0';
+                sval = va_arg ( ap, char * );
+                sprintf  ( msgBuf, frmToken, sval );
+                copy_buf ( &s, msgBuf, nSize, &nSize );
+                nState = IN_FORM;
+                break;
+
             case 's':
                 /*
                  * Ensure width or max length, if specified, does not exceed
@@ -236,12 +259,12 @@ Qvsnprintf ( char *s, size_t n, const char *format, va_list ap )
                     break;
                 }
                 
-                s_len = strlen ( sval );
+                s_len = Qstrlen ( sval, nSize );
                 
                 /*
                  * Make sure we have room if limit is unspecified.
                  */
-                if ( bLimit == FALSE && s_len > nSize )
+                if ( bLimit == FALSE && s_len >= nSize )
                     return -1;
                 
                 /* 
@@ -371,3 +394,24 @@ Qsprintf ( char *s, const char *format, ... )
     va_end   ( ap );
     return rslt;
 }
+
+
+/*
+ * This is a safe strlen().  It stops when it hits a null or
+ * when the maximum size has been reached.  This way you don't
+ * run off the end of the buffer looking for a null that might
+ * not be there.
+ */
+long Qstrlen ( const char *s, long m )
+{
+    long len = 0;
+
+
+    if ( s == NULL )
+        return 0;
+
+    while ( m-- > 0 && *s++ != '\0' )
+        len++;
+
+    return len;
+}
