@@ -5,13 +5,13 @@
      Laurence Lundblade <lgl@qualcomm.com>,
      Randy Gellens
 
-     Copyright 1997,1998,1999, 2000 QUALCOMM Incorporated.  All rights reserved.
+     Copyright 1997-2001, QUALCOMM Incorporated.  All rights reserved.
      The file License.txt specifies the terms for use, modification, and 
      redistribution.
 
      File: striphtml.c
      Version: 0.3.1, May 1999
-     Last Edited: 14 March 2000
+     Last Edited: 23 Januay 2001
 
   ---- */
      
@@ -296,8 +296,10 @@ enum HtmlElems {
    ELEM_XMP
 };
 
+#ifdef _TRAP_URLS
 static char gszURLchars[] = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
                             "0123456789+.-%#;/?:@=&$_*'()\n";
+#endif
 
 
 /*------------  Helper functions  -----------*/
@@ -324,19 +326,19 @@ static int FindE(char *szE, int what, int *pnLen)
    while (*szSrch) {
       char *sz = szE;
       while (what ? *szSrch == *sz : CharEQI(*szSrch, *sz)) {
-	 ++szSrch;
-	 ++sz;
-	 if (*szSrch == ';') {
+         ++szSrch;
+         ++sz;
+         if (*szSrch == ';') {
            if (what ? !CharIsAlnum(*sz) : !*sz) {
              *pnLen = sz - szE;
-	     return id;
-	    } else
-	       break;			/* no good */
-	 }
+             return id;
+            } else
+               break;                   /* no good */
+         }
       }
       
       while (*szSrch++ != ';')
-	 ;
+         ;
 
       ++id;
    }
@@ -368,7 +370,7 @@ static char FindEnt(char *szEnt, int *pnLen)
       int nNum = ScanInt((CSTR *)&szNum);
 
       if (*szNum == ';')
-	 ++szNum;
+         ++szNum;
 
       *pnLen = szNum - szEnt;
       return nNum;
@@ -397,17 +399,20 @@ static char FindEnt(char *szEnt, int *pnLen)
 #define DCLEAR()      (me->ndxData = 0)
 #define DTERMIN()     (me->szData[me->ndxData] = 0)
 #define DLENGTH()     (me->ndxData)
-#define DAPPEND(ch)   ((me->ndxData < DATAMAX-1) && (me->szData[me->ndxData++] = ch))
+#define DAPPEND(ch)   (void) ( (me->ndxData < DATAMAX-1) \
+                                && (me->szData[me->ndxData++] = ch) )
 
 #define AN_CLEAR()    (me->nattrName = 0)
 #define AN_TERMIN()   (me->attrName[me->nattrName] = 0)
 #define AN_LENGTH()   (me->nattrName)
-#define AN_APPEND(ch) ((me->nattrName < DATAMAX-1) && (me->attrName[me->nattrName++] = ch))
+#define AN_APPEND(ch) (void) ( (me->nattrName < DATAMAX-1) \
+                               && (me->attrName[me->nattrName++] = ch) )
 
 #define AV_CLEAR()    (me->nattrVal = 0)
 #define AV_TERMIN()   (me->attrVal[me->nattrVal] = 0)
 #define AV_LENGTH()   (me->nattrVal)
-#define AV_APPEND(ch) ((me->nattrVal < ATTRMAX-1) && (me->attrVal[me->nattrVal++] = ch))
+#define AV_APPEND(ch) (void) ( (me->nattrVal < ATTRMAX-1) \
+                                && (me->attrVal[me->nattrVal++] = ch) )
 
 
 #define OUTCH(ch)          (me->pfOut(me->pvOut, &(ch), 1))
@@ -427,6 +432,8 @@ static char FindEnt(char *szEnt, int *pnLen)
 static void outNewline(Html *me)
 {
   char tmp[3];
+  (void) me;
+
 
   StrCopy(tmp, lineEnd);
   OUTBUF(tmp, EOLLEN);
@@ -440,6 +447,9 @@ static void outNewline(Html *me)
 void textPlainToHTMLHead(void *state, char *szText, long len)
 {
     Html *me = (Html *)state;
+    (void) szText;
+    (void) len;
+
 
 #ifdef _DEBUG
    OUTBUF("<!---generated by textPlainToHTML---->\n", 39);
@@ -456,6 +466,10 @@ void textPlainToHTMLHead(void *state, char *szText, long len)
 void textPlainToHTMLTail(void *state, char *szText, long len)
 {
     Html *me = (Html *)state;
+    (void) szText;
+    (void) len;
+
+
     OUTBUF("</pre></html>\n", 14);
     return;
 }
@@ -469,6 +483,8 @@ void textPlainToHTML(void *state, char *szText, long len)
    Html *me = (Html *)state;
    char ch;
    CSTR pc = szText;
+   (void) len;
+
 
    if(szText == NULL) return;
 
@@ -513,6 +529,8 @@ void *textFlowedToHTMLInit(OutputFn oFn, void *oFnState)
 void textFlowedToHTMLHead(void *state, char *szText, long len)
 {
    struct Flowed2HTML *me = (struct Flowed2HTML *)state;
+   (void) szText;
+   (void) len;
 
 
 #ifdef _DEBUG
@@ -542,6 +560,8 @@ void textFlowedToHTMLHead(void *state, char *szText, long len)
 void textFlowedToHTMLTail(void *state, char *szText, long len)
 {
   struct Flowed2HTML *me = (struct Flowed2HTML *)state;
+  (void) szText;
+  (void) len;
 
 
 #ifdef _TRAP_URLS
@@ -584,7 +604,7 @@ void ffToHTML_lineStart(struct Flowed2HTML *me)
              
      int n;
      for (n = me->m_nQuoteDepth - me->m_nLastQuoteDepth; n; n--)
-       OUTBUF("<blockquote>", 12);
+       OUTBUF("<blockquote type=cite>", 12);
    }
            
    else 
@@ -612,6 +632,7 @@ void textFlowedToHTML(void *state, char *szText, long len)
    char ch;
    CSTR pc = szText;
    int  bCharProcessed  = FALSE;
+   (void) len;
 
 
    if(szText == NULL) return;
@@ -628,7 +649,7 @@ void textFlowedToHTML(void *state, char *szText, long len)
        
        if (me->m_cURLbuf[0] == '<' && ch == '>')
        {
-	 /* We've found the end of the URL */
+         /* We've found the end of the URL */
 
          /* Output the URL as a URL, then again encoded */
 
@@ -643,12 +664,12 @@ void textFlowedToHTML(void *state, char *szText, long len)
          j = me->m_nURLcount;
          OUTBUF("<A HREF=\"", 9);
          while (j && p && *p)
-	 {
+         {
            if (*p != '\n')
              OUTCH(*p);
            j--;
            p++;
-	 }
+         }
          OUTBUF("\">", 2);
 
          p = me->m_cURLbuf;
@@ -658,18 +679,18 @@ void textFlowedToHTML(void *state, char *szText, long len)
            switch(*p)
            {
              case '<':
-	     OUTBUF("&lt;",4);
-	     break;
+             OUTBUF("&lt;",4);
+             break;
       
              case '>':
              OUTBUF("&gt;", 4);
              break;
 
              case '&':
-	     OUTBUF("&amp;",5);
-	     break;
+             OUTBUF("&amp;",5);
+             break;
 
-	     case '\n':
+             case '\n':
              break;
   
              default:
@@ -699,7 +720,7 @@ void textFlowedToHTML(void *state, char *szText, long len)
 #endif
          
          if (me->m_cURLbuf[0] == '<')
-	 {
+         {
            start = 1;
            OUTBUF("&lt;", 4);
          }
@@ -727,7 +748,7 @@ void textFlowedToHTML(void *state, char *szText, long len)
      {
        switch (ch)
        {
-	 case '>':
+         case '>':
            me->m_nQuoteDepth++; /* count quote depth of this line */
            bCharProcessed = TRUE;
            break;
@@ -770,6 +791,9 @@ void textFlowedToHTML(void *state, char *szText, long len)
          case DASH_DASH_SPACE:
            me->m_nSigSepState = (ch == '\n') ? DASH_DASH_SPACE : NO_DASH;
            break;
+
+         case NO_DASH:
+           break;
        }
      }
 
@@ -788,8 +812,8 @@ void textFlowedToHTML(void *state, char *szText, long len)
          OUTBUF(" ", 1);
            
          if (ch == '\n' && me->m_nSigSepState != DASH_DASH_SPACE)
-	 { 
-	   me->m_bInPara = 1; /* Soft new line */
+         { 
+           me->m_bInPara = 1; /* Soft new line */
            me->m_nLineState = AT_START;
            me->m_nSigSepState = NO_DASH;
            bCharProcessed = TRUE;
@@ -819,22 +843,22 @@ void textFlowedToHTML(void *state, char *szText, long len)
        switch(ch) 
        {
          case '<':
-	   OUTBUF("&lt;",4);
-	   break;
+           OUTBUF("&lt;",4);
+           break;
 
          case '>':
            OUTBUF("&gt;",4);
            break;
 
          case '&':
-	   OUTBUF("&amp;",5);
-	   break;
+           OUTBUF("&amp;",5);
+           break;
 
          case '\n':
            if (me->m_bInPara)
              OUTBUF("<P>\n", 4);
            else
-	     OUTBUF("<BR>\n", 5);
+             OUTBUF("<BR>\n", 5);
 
            me->m_bInPara      = 0;
            me->m_nLineState   = AT_START;
@@ -843,8 +867,8 @@ void textFlowedToHTML(void *state, char *szText, long len)
            break;
   
          default:
-	   OUTCH( ch );
-	   break;
+           OUTCH( ch );
+           break;
        } /* switch */
 
      } /* bCharProcessed == FALSE */
@@ -862,6 +886,8 @@ void textHTMLToPlain(void *state, char *szText, long len)
    char ch;
    Html *me = (Html *)state;
    int   junk;
+   (void) len;
+
 
    if(szText == NULL) return;
 
@@ -871,140 +897,140 @@ void textHTMLToPlain(void *state, char *szText, long len)
 
       switch (me->nState) {
       case ssData:
-	 if (ch == '<') {
-	    DCLEAR();
-	    me->nState = ssTagOpen;
-	 }
+         if (ch == '<') {
+            DCLEAR();
+            me->nState = ssTagOpen;
+         }
 
-	 else if (ch == '&' && !me->szXMP[0]) {
-	    DCLEAR();
-	    me->nState = ssEntity;
-	 }
+         else if (ch == '&' && !me->szXMP[0]) {
+            DCLEAR();
+            me->nState = ssEntity;
+         }
 
-	 else
-	    TEXTCH(ch);
-	 break;
+         else
+            TEXTCH(ch);
+         break;
 
       case ssComment:
-	 if (ch == '>'
-	     && me->cntDashes >= 2) {/* standard SGML use "(me->cntDashes % 4) < 2" */
-	    /* comment terminated */
-	    me->nState = ssData;
-	    break;
-	 }
-	 if (ch == '-')
-	    ++me->cntDashes;
-	 else
-	    me->cntDashes = 0;
-	 break;
+         if (ch == '>'
+             && me->cntDashes >= 2) {/* standard SGML use "(me->cntDashes % 4) < 2" */
+            /* comment terminated */
+            me->nState = ssData;
+            break;
+         }
+         if (ch == '-')
+            ++me->cntDashes;
+         else
+            me->cntDashes = 0;
+         break;
 
-	 /*
-	 || ssTagOpen : First character following '<'
-	 */
+         /*
+         || ssTagOpen : First character following '<'
+         */
       case ssTagOpen:
-	 if (CharIsAlpha(ch) || ch == '!' || ch == '/') {
-	    /* Moscape takes only alpha, "!" and "/" as tag opening characters */
-	    /* All other "tags" will be passed through verbatim. */
-	    DAPPEND(ch);
-	    me->nState = ssTagName;
-	    AN_CLEAR();
-	 } else {
-	    TEXTBUF("<", 1);	/* ssTagOpen is entered only from ssData */
-	    me->nState = ssData;
-	    goto immediate_transition;
-	 }
-	 break;
+         if (CharIsAlpha(ch) || ch == '!' || ch == '/') {
+            /* Moscape takes only alpha, "!" and "/" as tag opening characters */
+            /* All other "tags" will be passed through verbatim. */
+            DAPPEND(ch);
+            me->nState = ssTagName;
+            AN_CLEAR();
+         } else {
+            TEXTBUF("<", 1);    /* ssTagOpen is entered only from ssData */
+            me->nState = ssData;
+            goto immediate_transition;
+         }
+         break;
 
 
-	 /*
-	 || ssTagName : Characters in a tag name
-	 */
+         /*
+         || ssTagName : Characters in a tag name
+         */
       case ssTagName:
-	 if (CharIsSgmlTok(ch)) {
-	    DAPPEND(ch);
-	 } else {
-	    DAPPEND(0);/* terminate the tag name, go to next buffer position */
+         if (CharIsSgmlTok(ch)) {
+            DAPPEND(ch);
+         } else {
+            DAPPEND(0);/* terminate the tag name, go to next buffer position */
 
-	    /*
-	    || If in XMP all tags should be ignored except a matching close tag. 
+            /*
+            || If in XMP all tags should be ignored except a matching close tag. 
             || In PLAINTEXT, all tags are ignored, period.
-	    */
-	    if (me->szXMP[0]
-	        && !(me->szData[0] == '/' && StrEQI(me->szData+1, me->szXMP))) {
-	        /* Not a matching close tag => spit it out as it was received. */
-		TEXTBUF("<", 1);
-		me->nState = ssData;
-		TEXTSZ(me->szData);
-		goto immediate_transition;         /* process ch also */
+            */
+            if (me->szXMP[0]
+                && !(me->szData[0] == '/' && StrEQI(me->szData+1, me->szXMP))) {
+                /* Not a matching close tag => spit it out as it was received. */
+                TEXTBUF("<", 1);
+                me->nState = ssData;
+                TEXTSZ(me->szData);
+                goto immediate_transition;         /* process ch also */
             }
-	    else if (StrBeginsI("!--", me->szData)) {
-	       /* Begin comment */
-	       me->cntDashes = 2;
-	       me->nState = ssComment;
+            else if (StrBeginsI("!--", me->szData)) {
+               /* Begin comment */
+               me->cntDashes = 2;
+               me->nState = ssComment;
 
-	       /* Process characters after "!--" that we've already received */
+               /* Process characters after "!--" that we've already received */
                /* [The only chars that could be of importance here are       */
                /* dashes; this ensures an accurate count.]                   */
-	       TEXTBUF(me->szData + 3, DLENGTH() - 4);
+               TEXTBUF(me->szData + 3, DLENGTH() - 4);
                       /* 4 = 3 for "!--"  +  1 for DAppend(0) */
-	       goto immediate_transition;	/* and process ch also */
-	    }
-	    goto enter_ssTagGap;
-	 } /* else ignore */
-	 break;
+               goto immediate_transition;       /* and process ch also */
+            }
+            goto enter_ssTagGap;
+         } /* else ignore */
+         break;
 
-	 /*
-	 || ssTagGap:  wait for attribute or end of tag
-	 */
+         /*
+         || ssTagGap:  wait for attribute or end of tag
+         */
 
       enter_ssTagGap:
-	 me->nState = ssTagGap;
-	 /* fallthrough */
+         me->nState = ssTagGap;
+         /* fallthrough */
       case ssTagGap:
-	 if (ch == '>') {
-	    /*
-	    || End of open or close tag
-	    */
-	    BOOL bClose = (me->szData[0] == '/');
-	    int id = FindE(me->szData + bClose, 0, &junk);
+         if (ch == '>') {
+            /*
+            || End of open or close tag
+            */
+            BOOL bClose = (me->szData[0] == '/');
+            int id = FindE(me->szData + bClose, 0, &junk);
 
-	    if (bClose)
-	       Html_CloseElem(me, id);
-	    else
-	       Html_OpenElem(me, id);
+            if (bClose)
+               Html_CloseElem(me, id);
+            else
+               Html_OpenElem(me, id);
 
-	    me->nState = ssData;
+            me->nState = ssData;
          } else if (CharIsSgmlTok(ch)) {
             AN_CLEAR();
             AV_CLEAR();
-	    goto enter_ssAttrName;
-	 }
-	 /* else ignore */
-	 break;
+            goto enter_ssAttrName;
+         }
+         /* else ignore */
+         break;
 
-	 /*
-	 || ssAttrName : get attribute name.  Entered when a token-start
+         /*
+         || ssAttrName : get attribute name.  Entered when a token-start
          || is encountered
-	 */
+         */
 
       enter_ssAttrName:
-	 me->nState = ssAttrName;
-	 /* fallthrough */
+         me->nState = ssAttrName;
+         /* fallthrough */
       case ssAttrName:
-	 if (CharIsSgmlTok(ch)) {
-	    /* part of the attribute name; */
-	    AN_APPEND(ch);
-	    break;
-	 } else if (ch > 32 && ch != '>' && ch != '=') {
-	    break;  /* ignore */
-	 }
+         if (CharIsSgmlTok(ch)) {
+            /* part of the attribute name; */
+            AN_APPEND(ch);
+            break;
+         } else if (ch > 32 && ch != '>' && ch != '=') {
+            break;  /* ignore */
+         }
 
-	 me->nState = ssAttrGap;
-	 /* fallthrough: goto immediate_transition to ssTagAttrGap */
+         me->nState = ssAttrGap;
+         /* fallthrough: goto immediate_transition to ssTagAttrGap */
 
       case ssAttrGap:
         AN_TERMIN();
-	if(AN_LENGTH() && (
+        if(AN_LENGTH() && (
             StrEQI(me->attrName, "alt") || StrEQI(me->attrName, "href"))){
           /* Got an attribute we care about */
           AV_CLEAR();
@@ -1012,38 +1038,38 @@ void textHTMLToPlain(void *state, char *szText, long len)
           AN_CLEAR();
         }
         if (ch == '=') {
-	  me->nState = ssAttrEqGap;
-	} else if (ch == '>') {
-	  goto enter_ssAttrComplete;
-	}
-	/* else ignore */
-	break;
+          me->nState = ssAttrEqGap;
+        } else if (ch == '>') {
+          goto enter_ssAttrComplete;
+        }
+        /* else ignore */
+        break;
 
-      case ssAttrEqGap:		/* after attr and "=" */
-	 if (ch ==  '"' || ch == '\'' || ch == '[') {
-	    me->chQuote = (ch == '[' ? ']' : ch);
-	    me->nState = ssValueQuoted;
-	 } else if (ch == '>') {
-	    goto enter_ssAttrComplete;
-	 } else if (ch > 32) {
-	    me->nState = ssValue;
-	 }
-	 break;
+      case ssAttrEqGap:         /* after attr and "=" */
+         if (ch ==  '"' || ch == '\'' || ch == '[') {
+            me->chQuote = (ch == '[' ? ']' : ch);
+            me->nState = ssValueQuoted;
+         } else if (ch == '>') {
+            goto enter_ssAttrComplete;
+         } else if (ch > 32) {
+            me->nState = ssValue;
+         }
+         break;
 
-	 /*
-	 || Inside an unquoted attribute value
-	 */
+         /*
+         || Inside an unquoted attribute value
+         */
       case ssValue:
-	 if (ch != '>' && ch > 32) {
-	   if(AN_LENGTH())
+         if (ch != '>' && ch > 32) {
+           if(AN_LENGTH())
           AV_APPEND(ch);
-	    break;
-	 }
-	 /* fallthrough */
+            break;
+         }
+         /* fallthrough */
 
-	 /*
-	 || ssAttrComplete : Complete the attribute and process ch
-	 */
+         /*
+         || ssAttrComplete : Complete the attribute and process ch
+         */
 
       enter_ssAttrComplete:
       case ssAttrComplete:
@@ -1051,63 +1077,63 @@ void textHTMLToPlain(void *state, char *szText, long len)
         AV_TERMIN();
      
       me->nState = ssTagGap;
-	 goto immediate_transition;
+         goto immediate_transition;
 
 
       case ssValueQuoted:
-	 if (ch == me->chQuote) {
-	    me->nState = ssAttrComplete;
-	 } else {
-	    /* character is part of the quoted value */
-	    if(AN_LENGTH())
+         if (ch == me->chQuote) {
+            me->nState = ssAttrComplete;
+         } else {
+            /* character is part of the quoted value */
+            if(AN_LENGTH())
           AV_APPEND(ch);
-	 }
-	 break;
+         }
+         break;
 
-	 /*
-	 || First character of entity or numeric reference (maybe)
-	 */
+         /*
+         || First character of entity or numeric reference (maybe)
+         */
       case ssEntity:
-	if (ch == '#' || CharIsAlpha(ch)) {
-	   /* Could be a valid entity */
-	    DAPPEND(ch);
-	    me->nState = ssEntityBody;
-	 } else {
-	    /* False alarm; regurgitate '&' */
-	    TEXTBUF("&", 1);
-	    me->nState = ssData;
-	    goto immediate_transition;	/* process ch right now */
-	 }
-	 break;
+        if (ch == '#' || CharIsAlpha(ch)) {
+           /* Could be a valid entity */
+            DAPPEND(ch);
+            me->nState = ssEntityBody;
+         } else {
+            /* False alarm; regurgitate '&' */
+            TEXTBUF("&", 1);
+            me->nState = ssData;
+            goto immediate_transition;  /* process ch right now */
+         }
+         break;
 
       case ssEntityBody:
-	 if (CharIsAlnum(ch) && me->ndxData <= 6) {
-	    DAPPEND(ch);
-	 } else {
-	    /* couldn't be a valid, properly-terminated entity (unless ch == ';') */
-	    DTERMIN();
-	    {
-	     int nLen = 0;
-	     char chEnt = FindEnt(me->szData, &nLen);
+         if (CharIsAlnum(ch) && me->ndxData <= 6) {
+            DAPPEND(ch);
+         } else {
+            /* couldn't be a valid, properly-terminated entity (unless ch == ';') */
+            DTERMIN();
+            {
+             int nLen = 0;
+             char chEnt = FindEnt(me->szData, &nLen);
 
-	     if (nLen)
-	        TEXTCH(chEnt);
-	     else
-	        TEXTBUF("&", 1);
+             if (nLen)
+                TEXTCH(chEnt);
+             else
+                TEXTBUF("&", 1);
 
-	     /* Output the characters we've read which aren't part of the entiy. */
-	     me->nState = ssData;
+             /* Output the characters we've read which aren't part of the entiy. */
+             me->nState = ssData;
 
-	     /* If there were any unused characers OR if ch is not an acceptable terminator, */
-	     /*    output those characters and the terminator. */
-	     if (nLen < me->ndxData || ch != ';') {
-	        while (nLen < me->ndxData)
-		   TEXTCH(me->szData[nLen++]);
-	        goto immediate_transition;
-	     }
-	    }
-	 }
-	 break;
+             /* If there were any unused characers OR if ch is not an acceptable terminator, */
+             /*    output those characters and the terminator. */
+             if (nLen < me->ndxData || ch != ';') {
+                while (nLen < me->ndxData)
+                   TEXTCH(me->szData[nLen++]);
+                goto immediate_transition;
+             }
+            }
+         }
+         break;
       }
    }
 }
@@ -1167,7 +1193,7 @@ static void Html_OpenElem(Html *me, int nTok)
       /* breaks==2 for these: */
    case ELEM_PLAINTEXT:
    case ELEM_XMP:
-      SETXMP(nTok == ELEM_XMP ? "XMP" : " ");	/* " " cannot be matched => PLAINTEXT never exits */
+      SETXMP(nTok == ELEM_XMP ? "XMP" : " ");   /* " " cannot be matched => PLAINTEXT never exits */
       /* FALLTHROUGH */
 
    case ELEM_PRE:
@@ -1240,45 +1266,45 @@ static void Html_Text(Html *me, CSTR szText, int cbText)
 
       switch (me->nTextState) {
       case tsInWord:
-	 if (CHARISSPACE(ch)) {
-	    OUTBUF(" ", 1);
-	    me->nTextState = tsInSpace;
-	 }
-	 else OUTCH(ch);
-	 break;
+         if (CHARISSPACE(ch)) {
+            OUTBUF(" ", 1);
+            me->nTextState = tsInSpace;
+         }
+         else OUTCH(ch);
+         break;
 
       case tsInSpace:
-	 if ( !CHARISSPACE(ch) ) {
-	    me->nBreaks = 0;
-	    OUTCH(ch);
-	    me->nTextState = tsInWord;
-	 }
-	 break;
+         if ( !CHARISSPACE(ch) ) {
+            me->nBreaks = 0;
+            OUTCH(ch);
+            me->nTextState = tsInWord;
+         }
+         break;
 
       case tsPre:
-	 if (ch == '\n') {
+         if (ch == '\n') {
             outNewline(me);
-	    me->nCol = 0;
-	 } else if (ch == '\t') {
-	    /* 
-	    ||     We are not expanding tabs, but nCol is still useful for determining whether
-	    ||     we are at the start of a line.
-	    || do
-	    ||   OUTCH(' ');
-	    || while ((++me->nCol % 8) != 0);
-	    */
-	    OUTCH(ch);
-	    ++me->nCol;
-	 } else if (((unsigned char)ch) >= 32) {
-	    OUTCH(ch);
-	    ++me->nCol;
-	 }
-	 break;
+            me->nCol = 0;
+         } else if (ch == '\t') {
+            /* 
+            ||     We are not expanding tabs, but nCol is still useful for determining whether
+            ||     we are at the start of a line.
+            || do
+            ||   OUTCH(' ');
+            || while ((++me->nCol % 8) != 0);
+            */
+            OUTCH(ch);
+            ++me->nCol;
+         } else if (((unsigned char)ch) >= 32) {
+            OUTCH(ch);
+            ++me->nCol;
+         }
+         break;
 
       case tsTitle:
       case tsComment:
-	 /* Ignore, for now. */
-	 break;
+         /* Ignore, for now. */
+         break;
 
       }
    }
@@ -1287,7 +1313,7 @@ static void Html_Text(Html *me, CSTR szText, int cbText)
 
 
 void *textHTMLInit(OutputFn oFn, void *oFnState, TextCharSetType partCharSet,
-		   TextCharSetType reqCharSet)  
+                   TextCharSetType reqCharSet)  
 {
   static Html html;
 
