@@ -1,10 +1,26 @@
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated.  All rights reserved.
+ * Copyright (c) 2001 QUALCOMM Incorporated.  All rights reserved.
  * The license.txt file specifies the terms for use, 
  * modification, and redistribution.
  *
  * Revisions:
  *
+ *     01/29/01  [rcg]
+ *             - Added check for "-no-mime".
+ *             - Now using p->bDo_timing instead of DO_TIMING.
+ *             - Now using p->grp_serv_mode and p->grp_no_serv_mode instead
+ *               of SERVER_MODE_GROUP_INCLUDE and SERVER_MODE_GROUP_EXCLUDE.
+ *             - Now using p->authfile instead of AUTHFILE.
+ *
+ *     01/18/01  [rcg]
+ *             - pwnam, run-time vs compile time, etc. changes.
+ *
+ *     11/14/00  [rcg]
+ *             - Trim domain name only if requested.
+ *
+ *     10/02/00  [rcg]
+ *             - Now calling check_config_files to check qpopper-options files.
+ *
  *     09/09/00  [rcg]
  *             - Now stripping domain name from user name.
  *
@@ -39,10 +55,9 @@
 
 #include <sys/types.h>
 #include "config.h"
+#include "md5.h"
 
-#ifdef DO_TIMING
-#  include <time.h>
-#endif /* DO_TIMING */
+#include <time.h>
 
 
 #ifdef APOP
@@ -79,14 +94,15 @@
 #include <flock.h>
 #include <stdlib.h>
 
-#include "genpath.h"
 #include "popper.h"
 #include "snprintf.h"
+#include "string_util.h"
 
 
 /*
  * Obscure password so a cleartext search doesn't come up with
- * something interesting.
+ * something interesting.  Also de-obscures an already obscured
+ * string.
  *
  */
 
@@ -122,6 +138,7 @@ POP *p;
                    digest[16];
     struct passwd *pw;
     int            j = 0;
+    size_t         user_name_len = 0;
 
 #ifdef GDBM
     char apop_file[BUFSIZ];
@@ -142,10 +159,7 @@ POP *p;
 
     MD5_CTX mdContext;
     int f;
-
-#ifdef DO_TIMING
     time_t my_timer = time(0);
-#endif /* DO_TIMING */
 
 #ifdef SCRAM_ONLY
     return ( pop_auth_fail ( p, POP_FAILURE, HERE,
@@ -156,41 +170,70 @@ POP *p;
     /* 
      * Downcase user name if requested 
      */
-    if ( p->downcase_user )
+    if ( p->bDowncase_user )
         downcase_uname ( p, p->pop_parm[1] );
 
     /* 
-     * strip domain name
+     * strip domain name if requested
      */
-    trim_domain ( p, p->pop_parm[1] );
+    if ( p->bTrim_domain )
+        trim_domain ( p, p->pop_parm[1] );
 
-    strncpy (p->user, p->pop_parm[1], sizeof(p->user) );
-    p->user [ sizeof(p->user)-1 ] = 0;
-    strcpy ( p->authid, p->user );  /* userid is also authentication id */
+    /*
+     * Save the user name 
+     */
+    user_name_len = strlen ( p->pop_parm[1] );
+    
+    if ( user_name_len > 8 
+         && !strcmp(p->pop_parm[1] + ( user_name_len - 8 ), "-no-mime" ) ) {
+        /* turn on mangling! */
+        p->bNo_mime     = TRUE;
+        user_name_len  -= 8;
+        DEBUG_LOG4 ( p, "Turned on MIME mangling; "
+                        "user name entered was (%u)'%.100s'; "
+                        "user name reset to (%u)'%.100s'",
+                     strlen(p->pop_parm[1]),   p->pop_parm[1],
+                     (unsigned) user_name_len, p->pop_parm[1] );
+      }
+    
+    if ( user_name_len > ( sizeof(p->user) -1 ) ) {
+        pop_log ( p, POP_WARNING, HERE, "Excessive user name length (%lu); "
+                                  "truncated to %lu: %.100s",
+                  (long unsigned) user_name_len, 
+                  (long unsigned) (sizeof(p->user) -1), 
+                  p->pop_parm[1] );
+        user_name_len = sizeof(p->user) -1;
+    }
+    
+    strlcpy ( p->user,   p->pop_parm[1], sizeof(p->user) );
+
+#ifdef    SCRAM
+    strlcpy ( p->authid, p->user, sizeof(p->authid) );  /* userid is also authentication id */
+#endif /* SCRAM */
 
     pop_log ( p, POP_INFO, HERE, "apop \"%s\"", p->user );
 
-#ifdef AUTHFILE
-    if ( checkauthfile ( p->user ) != 0 ) {
-        DEBUG_LOG2 ( p, "User %.128s not in authfile %.256s",
-                     p->user, AUTHFILE );
-        return ( pop_msg ( p, POP_FAILURE, HERE,
-                           "[AUTH] Permission denied.", p->user ) );
+    if ( p->authfile != NULL && checkauthfile ( p ) != 0 ) {
+        return pop_msg ( p, POP_FAILURE, HERE, "[AUTH] Permission denied." );
     }
-#endif /* AUTHFILE */
 
-#ifdef NONAUTHFILE
-    if ( checknonauthfile ( p->user ) != 0 ) {
-        DEBUG_LOG2 ( p, "User %.128s in nonauthfile %.256s",
-                     p->user, NONAUTHFILE );
-        return ( pop_msg ( p, POP_FAILURE, HERE,
-                           "[AUTH] Permission denied.", p->user ) );
+    if ( p->nonauthfile != NULL && checknonauthfile ( p ) != 0 ) {
+        return pop_msg ( p, POP_FAILURE, HERE, "[AUTH] Permission denied." );
     }
-#endif /* NONAUTHFILE */
 
-    pw = getpwnam ( p->user );
-    if ( ( pw == NULL) || (  pw->pw_passwd == NULL )
-                       || ( *pw->pw_passwd == '\0' ) ) {
+    /*
+     * Cache passwd struct for use later; this memory gets freed at the end
+     * of the session.
+     */
+    pw = getpwnam ( p->user );               /* get pointer to info */
+    if ( pw != NULL ) {
+        p->pw        = *pw;                  /* copy it */
+        p->pw.pw_dir = strdup(pw->pw_dir);   /* copy home directory */
+        DEBUG_LOG2 ( p, "home (%d): '%s'",
+                     strlen(p->pw.pw_dir), p->pw.pw_dir );
+    }
+    if ( ( pw == NULL ) || (  pw->pw_passwd == NULL )
+                        || ( *pw->pw_passwd == '\0' ) ) {
         return ( pop_auth_fail ( p, POP_FAILURE, HERE, ERRMSG_PW, p->user ) );
     }
 
@@ -209,18 +252,17 @@ POP *p;
                                  p->user, strerror(errno), errno ) );
 
 #ifdef GDBM
-    strncpy ( apop_file, APOP, sizeof(apop_file) - 1 );
-    apop_file [ sizeof(apop_file) -1 ] = '\0';
+    strlcpy ( apop_file, APOP, sizeof(apop_file) - 1 );
     j = stat ( apop_file, &st  );
     if ( ( j != -1 )
          && ( ( st.st_mode & 0777 ) != 0600 ) ) {
         gdbm_close ( db );
         DEBUG_LOG4 ( p, "stat returned %d; %s mode is %o (%x)",
-                     j, apop_file, st.st_mode, st.st_mode );
+                     j, apop_file, (int) st.st_mode, (int) st.st_mode );
         return ( pop_auth_fail ( p, POP_FAILURE, HERE,
                                  "[SYS/PERM] POP authentication DB "
                                  "has wrong mode (%.3o)",
-                                 st.st_mode & 0777 ) );
+                                 (unsigned int) st.st_mode & 0777 ) );
     }
     f = open ( apop_file, O_RDONLY );
     if ( f == -1 ) {
@@ -232,7 +274,7 @@ POP *p;
                                  strerror(e), e ) );
     }
 #else  
-    strncpy ( apop_dir, APOP, sizeof(apop_dir) - 5 );
+    strlcpy ( apop_dir, APOP, sizeof(apop_dir) - 5 );
 #  if defined(BSD44_DBM)
     strcat ( apop_dir, ".db" );
 #  else
@@ -290,7 +332,7 @@ POP *p;
 
     /*
      * (n)dbm invalidates the datum pointer after dbm_close (gdbm
-     * returns a copy), so we copy and obscure it before the close.
+     * returns a copy), so we copy and de-obscure it before the close.
      */
     apop_str = obscure ( ddatum.dptr );
 
@@ -330,56 +372,33 @@ POP *p;
 
     DEBUG_LOG1 ( p, "APOP authentication ok for \"%s\"", p->user );
 
-#ifdef DO_TIMING
-    p->login_time = time(0) - my_timer;
-#endif /* DO_TIMING */
+    if ( p->bDo_timing )
+        p->login_time = time(0) - my_timer;
 
     /*
      * Check if server mode should be set or reset based on group membership.
      */
 
-#ifdef SERVER_MODE_GROUP_INCL
-    if ( check_group ( p, pw, SERVER_MODE_GROUP_INCL ) ) {
+    if ( p->grp_serv_mode != NULL
+         && check_group ( p, pw, p->grp_serv_mode ) ) {
         p->server_mode = TRUE;
-    DEBUG_LOG2 ( p, "Set server mode for user %s; member of \"%.128s\" group",
-                 p->user, SERVER_MODE_GROUP_INCL );
+        DEBUG_LOG2 ( p, "Set server mode for user %s; "
+                        "member of \"%.128s\" group",
+                     p->user, p->grp_serv_mode );
     }
-#endif /* SERVER_MODE_GROUP_INCL */
 
-#ifdef SERVER_MODE_GROUP_EXCL
-    if ( check_group ( p, pw, SERVER_MODE_GROUP_EXCL ) ) {
+    if ( p->grp_no_serv_mode != NULL
+         && check_group ( p, pw, p->grp_no_serv_mode ) ) {
         p->server_mode = FALSE;
-    DEBUG_LOG2 ( p, "Set server mode OFF for user %s; member of \"%.128s\" group",
-                 p->user, SERVER_MODE_GROUP_INCL );
+        DEBUG_LOG2 ( p, "Set server mode OFF for user %s; "
+                        "member of \"%.128s\" group",
+                     p->user, p->grp_no_serv_mode );
     }
-#endif /* SERVER_MODE_GROUP_EXCL */
 
     /*
-     * Process ~/.qpopper-options file, if requested and present.
+     * Process qpopper-options files, if requested and present.
      */
-    if ( p->user_opts ) {
-        int         rslt;
-        char        buf [ 256 ];
-        struct stat stat_buf;
-
-        rslt = Qsnprintf ( buf, sizeof(buf), "%s/.qpopper-options", pw->pw_dir );
-        if ( rslt == -1 )
-            pop_log ( p, POP_PRIORITY, HERE, 
-                      "Unable to build user options file name for user %s",
-                      p->user );
-        else {
-            rslt = stat ( buf, &stat_buf );
-            if ( rslt == 0 ) {
-                rslt = pop_config ( p, buf, TRUE );
-                if ( rslt == POP_FAILURE ) {
-                    pop_log ( p, POP_PRIORITY, HERE,
-                              "Unable to process user options file for user %s",
-                              p->user );
-                }
-            }
-        }
-    } /* p->user_opts */
-
+    check_config_files ( p, pw );
 
     /*  
      * Make a temporary copy of the user's maildrop 
@@ -399,18 +418,16 @@ POP *p;
     /*  
      * Authorization completed successfully 
      */
-#ifdef LOG_LOGIN
-    pop_log ( p, POP_PRIORITY, HERE,
-              "(v%s) POP login by user \"%s\" at (%s) %s",
-              VERSION, p->user, p->client, p->ipaddr );
-#endif /* LOG_LOGIN */
+    if ( p->pLog_login != NULL )
+        do_log_login ( p );
+
 
 #ifdef DRAC_AUTH
     drac_it ( p );
 #endif /* DRAC_AUTH */
 
     return ( pop_msg ( p, POP_SUCCESS, HERE,
-                       "%s has %d visible message%s (%d hidden) in %d octets.",
+                       "%s has %d visible message%s (%d hidden) in %ld octets.",
                         p->user,
                         p->visible_msg_count,
                         p->visible_msg_count == 1 ? "" : "s",
