@@ -1,3 +1,4 @@
+
 /*
  * Copyright (c) 1989 Regents of the University of California.
  * All rights reserved.  The Berkeley software License Agreement
@@ -7,12 +8,44 @@
 
 
 /*
- * Copyright (c) 2000 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2001 Qualcomm Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  *
  * Revisions:
  *
+ *     02/12/01 [RCG]
+ *              - Save trace file name.
+ *
+ *     02/08/01 [RCG]
+ *              - Setting p->nMaxBulls from compile-time NEWBULLCNT.
+ *
+ *     02/03/01 [RCG]
+ *              - Now logging to p->pop_facility.
+ *
+ *     01/25/01 [RCG]
+ *              - Setting p->chunky_writes from compile-time default.
+ *
+ *     01/23/01 [RCG]
+ *              - login_delay and expire now in p.
+ *              - Initialize new -p- variables from compile-time values:
+ *                bDo_timing, bCheck_old_spool_loc, bCheck_hash_dir,
+ *                bCheck_pw_max, bUpdate_status_hdrs, bUpdate_on_abort,
+ *                bAuto_delete, bGroup_bulls, hash_spool, bHome_dir_mail,
+ *                bOld_style_uid, bUW_kluge, bKeep_temp_drop, grp_serv_mode,
+ *                grp_no_serv_mode, authfile, nonauthfile, bShy,
+ *                pMail_command, pCfg_spool_dir, pCfg_temp_dir,
+ *                pCfg_temp_name, pCfg_cache_dir, pCfg_cache_name.
+ *
+ *     12/21/00  [rcg]
+ *              - Delete 'a' and change 'l' to integer type.
+ *
+ *     11/14/00  [rcg]
+ *              - Added '-C' to trim domain name.
+ *
+ *     10/12/00  [rcg]
+ *              - Fitted LGL's TLS patches.
+ *
  *     09/30/00  [rcg]
  *              - Fixed tracing in stand-alone mode.
  *
@@ -144,8 +177,6 @@
 #endif /* HAVE_HSTRERROR */
 
 extern int      errno;
-extern int      login_delay;
-extern int      expire;
        int      no_rev_lookup;      /*  Avoid reverse lookup? */
 
 /*
@@ -192,7 +223,7 @@ kerb_authenticate ( p, addr )
     char           version[9];
     int            auth;
   
-    if ( p->kerberos ) {
+    if ( p->bKerberos ) {
 
 #    ifdef  K_GETSOCKINST
         k_getsockinst ( 0, instance );
@@ -221,7 +252,7 @@ kerb_authenticate ( p, addr )
         strncpy ( p->user, kdata.pname, sizeof(p->user) );
         p->AuthType = kerberos;
 
-    } /* p->kerberos */
+    } /* p->bKerberos */
 #  endif /* KRB4 */
 
 #  ifdef KRB5
@@ -236,7 +267,7 @@ kerb_authenticate ( p, addr )
     krb5_int32         auth_sys      = 0;
     int                sock          = 0;
 
-    if ( p->kerberos ) {
+    if ( p->bKerberos ) {
         retval = krb5_init_context ( &pop_context );
         if ( retval ) {
             pop_msg ( p, POP_FAILURE, HERE,
@@ -393,6 +424,7 @@ kerb_authenticate ( p, addr )
 /* 
  *  init:   Start a Post Office Protocol session
  */
+FILE *g_pTrace = NULL;
 
 int
 pop_init ( p, argcount, argmessage )
@@ -410,7 +442,7 @@ char    **      argmessage;
     int                     options = 0;
     int                     sp = 0;             /*  Socket pointer */
     int                     pw_handling = 0;    /*  How clear text passwords handled */
-    char                *   trace_file_name;
+    int                     tls_support = 0;    /*  Desired TLS/SSL support */
     struct hostent      *   hp = NULL;
 
 
@@ -431,29 +463,21 @@ char    **      argmessage;
      * Note that we zeroed the block above, so we really only need to
      * set any desired non-zero/non-NULL values.
      */
-    p->msgs_deleted        = 0;
-    p->last_msg            = 0;
-    p->bytes_deleted       = 0;
-    p->drop_size           = 0;
-    p->mmdf_separator      = NULL;
     p->bulldir             = BULLDIR;
-    p->dirty               = FALSE;
-    p->xmitting            = FALSE;
     p->AuthType            = noauth;
     p->AuthState           = none;
-    p->kerberos            = FALSE;
     p->AllowClearText      = ClearTextDefault;
     p->check_lock_refresh  = 5000; /* just a wild guess */
 
-#ifdef  KERBEROS
+#ifdef     KERBEROS
     p->kerberos_service    = KERBEROS_SERVICE;
+    DEBUG_LOG1 ( p, "Set kerberos_service to %s", p-.kerberos_service );
 #endif  /* KERBEROS */
 
-#ifdef SERVER_MODE
-    p->server_mode         = 1;
-#else
-    p->server_mode         = 0;
-#endif
+#ifdef    SERVER_MODE
+    p->server_mode         = TRUE;
+    DEBUG_LOG0 ( p, "Server mode now on by default for all users" );
+#endif /* SERVER_MODE */
 
 #ifdef    BULLDB
 #  ifdef    HAVE_USLEEP
@@ -461,12 +485,178 @@ char    **      argmessage;
 #  else
     p->bulldb_max_tries    = 10; /* when sleeping for seconds */
 #  endif /* HAVE_USLEEP */
+    DEBUG_LOG1 ( p, "Set bulldb_max_tries to %d", p->bulldb_max_tries );
 #endif /* BULLDB */
 
-#ifdef AUTO_DELETE
-    expire                 = 0;
+#ifdef    USE_BULL_GROUPS
+    p->bGroup_bulls        = TRUE;
+    DEBUG_LOG0 ( p, "Using group bulletins" );
+#endif /* USE_BULL_GROUPS */
+
+#ifdef    AUTO_DELETE
+    p->expire              = 0;
+    p->bAuto_delete        = TRUE;
+    DEBUG_LOG0 ( p, "Auto-expiring messages" );
 #endif /* AUTO_DELETE */
 
+#ifdef    FAST_UPDATE
+    p->bFast_update         = TRUE;
+    DEBUG_LOG0 ( p, "Set fast-update" );
+#endif /* FAST_UPDATE */
+
+#ifdef    DO_TIMING
+    p->bDo_timing          = TRUE;
+    DEBUG_LOG0 ( p, "Generating timing records" );
+#endif /* DO_TIMING */
+
+#ifndef       DONT_CHECK_OLD_SPOOL
+    p->bCheck_old_spool_loc= TRUE;
+#else
+    DEBUG_LOG0 ( p, "Omitting check for old temp files in old locations" );
+#endif /* not DONT_CHECK_OLD_SPOOL */
+
+#ifndef       DONT_CHECK_HASH_SPOOL_DIR
+    p->bCheck_hash_dir     = TRUE;
+#else
+    DEBUG_LOG0 ( p, "Omitting check for hashed spool directories" );
+#endif /* not DONT_CHECK_HASH_SPOOL_DIR */
+
+#ifdef    CHECK_UW_KLUDGE
+    p->bUW_kluge           = TRUE;
+    DEBUG_LOG0 ( p, "Checking for and hiding UW folder status messages" );
+#endif /* CHECK_UW_KLUDGE */
+
+#ifndef       DONT_CHECK_SP_MAX
+    p->bCheck_pw_max       = TRUE;
+#else
+    DEBUG_LOG0 ( p, "Omitting check for expired passwords" );
+#endif /* not DONT_CHECK_SP_MAX */
+
+#ifdef    KEEP_TEMP_DROP
+    p->bKeep_temp_drop     = TRUE;
+    DEBUG_LOG0 ( p, "Keeping temp drop files around" );
+#endif /* KEEP_TEMP_DROP */
+
+#ifndef       NO_STATUS
+    p->bUpdate_status_hdrs = TRUE;
+#else
+    DEBUG_LOG0 ( p, "Will not generate/update Status/UIDL headers" );
+#endif /* not NO_STATUS */
+
+#ifdef    HASH_SPOOL
+    p->hash_spool          = HASH_SPOOL;
+    DEBUG_LOG1 ( p, "Using hashed spool dirs; method %d", p->hash_spool );
+#endif /* HASH_SPOOL */
+
+#ifdef    HOMEDIRMAIL
+    p->pHome_dir_mail      = HOMEDIRMAIL;
+    DEBUG_LOG1 ( p, "Spool is %s in user's home dir", p->pHome_dir_mail );
+#endif /* HOMEDIRMAIL */
+
+#ifdef    OLD_STYLE_UIDL
+    p->bOld_style_uid      = TRUE;
+    DEBUG_LOG0 ( p, "Generating old (pre-3.x) style UIDs" );
+#endif /* OLD_STYLE_UIDL */
+
+#ifndef       NOUPDATEONABORT
+    p->bUpdate_on_abort    = TRUE;
+#else
+    DEBUG_LOG0 ( p, "Will not enter UPDATE state on abort" );
+#endif /* not NOUPDATEONABORT */
+
+#ifdef    SERVER_MODE_GROUP_INCL
+    p->grp_serv_mode       = SERVER_MODE_GROUP_INCL;
+    DEBUG_LOG1 ( p, "Server mode on by default for users in group %s",
+                 p->grp_serv_mode );
+#endif /* SERVER_MODE_GROUP_INCL */
+
+#ifdef    SERVER_MODE_GROUP_EXCL
+    p->grp_no_serv_mode    = SERVER_MODE_GROUP_EXCL;
+    DEBUG_LOG1 ( p, "Server mode off by default for users in group %s",
+                 p->grp_no_serv_mode );
+#endif /* SERVER_MODE_GROUP_EXCL */
+
+#ifdef    AUTHFILE
+    p->authfile            = AUTHFILE;
+    DEBUG_LOG1 ( p, "Permitting access only to users listed in %s",
+                 p->authfile );
+#endif /* AUTHFILE */
+
+#ifdef    NONAUTHFILE
+    p->nonauthfile         = NONAUTHFILE;
+    DEBUG_LOG1 ( p, "Denying access to users listed in %s", p->nonauthfile );
+#endif /* NONAUTHFILE */
+
+#ifdef    SHY
+    p->bShy                = TRUE;
+    DEBUG_LOG0 ( p, "Hiding version" );
+#endif /* SHY */
+
+#ifdef    MAIL_COMMAND
+    p->pMail_command       = MAIL_COMMAND;
+    DEBUG_LOG1 ( p, "Mail command (for XTND XMIT) is '%s'", p->pMail_command );
+#endif /* MAIL_COMMAND */
+
+/* Location of the spool directory */
+#ifdef    POP_MAILDIR
+    p->pCfg_spool_dir      = POP_MAILDIR;
+    DEBUG_LOG1 ( p, "Spool directory: %s", p->pCfg_spool_dir );
+#endif /* POP_MAILDIR */
+
+/* Location of the temporary mail drop directory */
+#ifdef    POP_DROP_DIR
+    p->pCfg_temp_dir       = POP_DROP_DIR;
+    DEBUG_LOG1 ( p, "Temp drop directory: %s", p->pCfg_temp_dir );
+#endif /* POP_DROP_DIR */
+
+/* The temporary mail drop file name constant */
+#ifdef    POP_DROP
+    p->pCfg_temp_name      = POP_DROP;
+    DEBUG_LOG1 ( p, "Temp drop file name: %s", p->pCfg_temp_name );
+#endif /* POP_DROP */
+
+/* Location of the cache directory */
+#ifdef    POP_CACHE_DIR
+    p->pCfg_cache_dir      = POP_CACHE_DIR;
+    DEBUG_LOG1 ( p, "Cache directory: %s", p->pCfg_cache_dir );
+#endif /* POP_CACHE_DIR */
+
+/* The cache file name constant */
+#ifdef    POP_CACHE
+    p->pCfg_cache_name     = POP_CACHE;
+    DEBUG_LOG1 ( p, "Cache file name: %s", p->pCfg_cache_dir );
+#endif /* POP_CACHE */
+
+#ifdef    CHUNKY_WRITES
+    p->chunky_writes       = (chunky_type) CHUNKY_WRITES;
+    DEBUG_LOG1 ( p, "Set chunky_writes to %d", (int) p->chunky_writes );
+#endif /* CHUNKY_WRITES */
+
+#ifdef    NO_ATOMIC_OPEN
+    p->bNo_atomic_open     = TRUE;
+    DEBUG_LOG0 ( p, "Set no_automic_open" );
+#endif /* NO_ATOMIC_OPEN */
+
+#ifdef    NEWBULLCNT
+    p->nMaxBulls           = NEWBULLCNT;
+#else
+    p->nMaxBulls           = 1;
+#endif /* NEWBULLCNT */
+    DEBUG_LOG1 ( p, "Set max-bulletins to %d", p->nMaxBulls );
+
+#ifndef   POP_FACILITY
+#  if defined(OSF1) || defined(LINUX)
+#    define POP_FACILITY    LOG_MAIL
+#  else
+#    define POP_FACILITY    LOG_LOCAL0
+#  endif /* OSF1 or Linux */
+#endif /* POP_FACILITY not defined */
+
+    p->log_facility        = (log_facility_type) POP_FACILITY;
+
+#ifdef    LOG_LOGIN
+    p->pLog_login = "(v%0) POP login by user \"%1\" at (%2) %3";
+#endif /* LOG_LOGIN */
 
     StackInit ( & ( p->InProcess ) );
     
@@ -481,85 +671,85 @@ char    **      argmessage;
 #ifdef SYSLOG42
     openlog ( p->myname, 0 );
 #else
-    openlog ( p->myname, POP_LOGOPTS, POP_FACILITY );
+    openlog ( p->myname, POP_LOGOPTS, p->log_facility );
 #endif
 
     /*  
      * Process command line arguments 
      */
-    while ( ( c = getopt ( argcount, argmessage, "b:BcdD:e:f:kK:L:p:RsSt:T:u") ) != EOF )
+    while ( ( c = getopt ( argcount, argmessage, "b:BcCdD:e:f:FkK:l:L:p:RsSt:T:uUvy:") ) != EOF )
         switch ( c ) {
 
-        /* Bulletins requested */
-        case 'b':
+        case 'b': /* Bulletins requested */
             p->bulldir = optarg;
             DEBUG_LOG1 ( p, "bulletin directory = %s", p->bulldir );
             break;
 
 #ifdef    BULLDB
-        /* Allow session to proceed without bulletin DB */
-        case 'B':
+        case 'B': /* Allow session to proceed without bulletin DB */
             p->bulldb_nonfatal = TRUE;
             DEBUG_LOG0 ( p, "Sessions can proceed without bulletin db" );
             break;
 #endif /* BULLDB */
 
-        /* Force user names to be lower case */
-        case 'c':
-            p->downcase_user = TRUE;
+        case 'c': /* Force user names to be lower case */
+            p->bDowncase_user = TRUE;
             DEBUG_LOG0 ( p, "user names will be downcased (-c)" );
             break;
 
-        /*  
-         * Debugging requested.
-         */
-        case 'd':
+        case 'C': /* Trim domain name from user name */
+            p->bTrim_domain = TRUE;
+            DEBUG_LOG0 ( p, "trimming domain names from user names (-C)" );
+            break;
+
+        case 'd': /* Debugging requested */
             p->debug = TRUE;
             options |= SO_DEBUG;
             DEBUG_LOG0 ( p, "Debugging turned on (-d)" );
             break;
 
 #ifdef    DRAC_AUTH
-        /* drac host specified */
-        case 'D':
+        case 'D': /* drac host specified */
             p->drac_host = optarg;
             DEBUG_LOG1 ( p, "drac host is %.64s", p->drac_host );
             break;
 #endif /* DRAC_AUTH */
 
-        /* Pop Extensions */
-        /* -e login_delay=xx,expire=xx */
-        case 'e':
-            {
+        case 'e': /* Pop Extensions */
+            {     /* -e login_delay=xx,expire=xx */
                 char *extn_options[] = { "login_delay", "expire", NULL };
                 char *value, *option = optarg;
                 while ( *option ) {
                     switch ( get_sub_option ( &option, extn_options, &value ) ) {
                     case 0:
-                        login_delay = value ? atoi ( value ) : -1;
-                        DEBUG_LOG1 ( p, "announcing login_delay %d", login_delay );
+                        p->login_delay = value ? atoi ( value ) : -1;
+                        DEBUG_LOG1 ( p, "announcing login_delay %d", p->login_delay );
                         break;
                     case 1:
-                        expire = value ? atoi ( value ) : -1;
-                        DEBUG_LOG1 ( p, "announcing expire %d", expire );
+                        p->expire = value ? atoi ( value ) : -1;
+                        DEBUG_LOG1 ( p, "announcing expire %d", p->expire );
                         break;
                     }
                 }
             }
             break;
 
-        /* Get options from a file */
-        case 'f':
-            if ( pop_config ( p, optarg, FALSE ) == POP_FAILURE ) {
+        case 'f': /* Get options from a file */
+            if ( pop_config ( p, optarg, CfgInit ) == POP_FAILURE ) {
                 fprintf ( stderr, "Unable to process config file %s\n",
                           optarg );
                 EXIT ( 1 );
             }
             break;
 
+        case 'F': /* Set fast-update */
+            p->bFast_update = TRUE;
+            DEBUG_LOG0 ( p, "set fast-update" );
+            break;
+
 #ifdef KERBEROS
         case 'k':
-            p->kerberos = TRUE;
+            p->bKerberos = TRUE;
             DEBUG_LOG0 ( p, "Using Kerberos (-k)" );
             break;
 
@@ -570,16 +760,36 @@ char    **      argmessage;
             break;
 #endif  /* KERBEROS */
 
-        /*  Touch the mail lock every this many messages (crude)  */
-        case 'L':
+
+        case 'l': /* TLS/SSL handling */
+            tls_support = atoi ( optarg );
+            switch ( tls_support ) {
+                case 0:
+                    p->tls_support = QPOP_TLS_NONE;
+                    break;
+                case 1:
+                    p->tls_support = QPOP_TLS_STLS;
+                    break;
+                case 2:
+                    p->tls_support = QPOP_TLS_ALT_PORT;
+                    break;
+                default:
+                    fprintf ( stderr, "Unrecognized -l value; 0 = default; "
+                                      "1 = stls; 2 = alternate-port\n" );
+                    errflag++;
+            }
+            DEBUG_LOG1 ( p, "tls-support=%d (-l)", tls_support );
+            break;
+
+        case 'L': /*  Check if we need to touch the mail lock every
+                   *  this many messages (crude)  */
             p->check_lock_refresh = atoi(optarg);
             DEBUG_LOG1 ( p, "Checking for maillock refresh every %i msgs", 
                          p->check_lock_refresh );
             break;
 
 #if defined(APOP) || defined(SCRAM)
-        /*  Set clear text password handling */
-        case 'p':
+        case 'p': /*  Set clear text password handling */
             pw_handling = atoi ( optarg );
             switch ( pw_handling ) {
                 case 0:
@@ -605,26 +815,22 @@ char    **      argmessage;
             break;
 #endif /* APOP or SCRAM */
 
-        /*  Avoid reverse lookups on client */
-        case 'R':
+        case 'R': /*  Omit reverse lookups on client IP */
             no_rev_lookup = TRUE;
             DEBUG_LOG0 ( p, "Avoiding reverse lookups (-R)" );
             break;
 
-        /* Stats requested */
-        case 's':
-            p->stats = TRUE;
+        case 's': /* Stats requested */
+            p->bStats = TRUE;
             DEBUG_LOG0 ( p, "Will generate stats records (-s)" );
             break;
 
-        /* Server mode */
-        case 'S':
+        case 'S': /* Server mode */
                 p->server_mode = TRUE;
                 DEBUG_LOG0 ( p, "server mode is the default (-S)" );
                 break;
 
-        /*  Debugging trace file specified */
-        case 't':
+        case 't': /*  Debugging trace file specified */
             p->debug = TRUE;
 
 #ifdef STANDALONE
@@ -644,32 +850,51 @@ char    **      argmessage;
                           optarg, STRERROR(errno), errno );
                 EXIT ( 1 );
             }
-            trace_file_name = optarg;
+            p->trace_name = optarg;
             DEBUG_LOG1 ( p, "Trace and Debug destination is file \"%s\"",
-                         trace_file_name );
+                         p->trace_name );
             break;
 
-        /*  Timeout value passed.  Default changed */
-        case 'T':
+        case 'T': /*  Timeout value passed.  Default changed */
             pop_timeout = atoi ( optarg );
-            DEBUG_LOG1(p, "timeout = %i (-T)", pop_timeout );
+            DEBUG_LOG1 ( p, "timeout = %i (-T)", pop_timeout );
             break;
 
-        /* Use ~/.qpopper-options file */
-        case 'u':
-            p->user_opts = TRUE;
+        case 'u': /* Use ~/.qpopper-options file */
+            p->bUser_opts = TRUE;
             DEBUG_LOG0 ( p, "Will use ~/qpopper-options file (-u)" );
             break;
 
-        /*  Unknown option received */
-        default:
+        case 'U': /* Use .qpopper-options file in spool location */
+            p->bSpool_opts = TRUE;
+            DEBUG_LOG0 ( p, "Will use .qpopper-options file in spool "
+                            "directory" );
+            break;
+
+        case 'v': /* RESERVED (handled in main.c) -- DO NOT USE */
+            break;
+
+        case 'y': /* log facility */
+            if ( set_option ( p, "log-facility", optarg ) == POP_SUCCESS ) {
+                closelog();
+#ifdef SYSLOG42
+                openlog ( p->myname, 0 );
+#else
+                openlog ( p->myname, POP_LOGOPTS, p->log_facility );
+#endif
+            } else {
+                fprintf ( stderr, "Error setting '-y' to %s\n", optarg );
+                errflag++;
+            }
+
+        default: /*  Unknown option received */
             errflag++;
         }
 
     /*  Exit if bad options specified */
 
 #ifdef AUTO_DELETE
-    if ( expire != 0 ) {
+    if ( p->expire != 0 ) {
         fprintf ( stderr, "EXPIRE must be 0 when AUTO_DELETE defined.\n" );
         errflag++;
     }
@@ -678,54 +903,84 @@ char    **      argmessage;
     if ( errflag ) {
 
 #ifdef    BULLDB
-        char *xB = " [-B]";
+        char *xB = "[-B] ";
 #else
         char *xB = "";
 #endif /* BULLDB */
 
 #ifdef    DRAC_AUTH
-        char *xD = " [-D drac-host]";
+        char *xD = "[-D drac-host] ";
 #else
         char *xD = "";
 #endif /* DRAC_AUTH */
 
 #if defined(APOP) || defined(SCRAM)
-        char *xp = " [-p 0-3]";
+        char *xp = "[-p 0-4] ";
 #else
         char *xp = "";
 #endif /* APOP or SCRAM */
 
 #ifdef KERBEROS
-        char *xk = " [-k]";
-        char *xK = " [-K service]";
+        char *xk = "[-k] ";
+        char *xK = "[-K service] ";
 #else
         char *xk = "";
         char *xK = "";
 #endif /* KERBEROS */
 
+#ifdef QPOP_SSL
+        char *xl = "[-l 0-2] ";
+#else
+        char *xl = "";
+#endif /* QPOP_SSL */
+
 
         fprintf ( stderr, "Usage: %s "
                           "[-b bulldir] "
-                          "%s"         /* bulldb_nonfatal, if BULLDB set */
+                          "%s"         /* -B / bulldb_nonfatal, if BULLDB set */
                           "[-c] "
+                          "[-C] "
                           "[-d] "
                           "[-e login_delay=nn,expire=nn] "
                           "[-f config-file] "
-                          "%s"          /* drac-host, if DRAC set */
-                          "%s"          /* kerberos, if KERBEROS set */
-                          "%s"          /* kerberos service, if KERBEROS set */
+                          "[-F] "
+                          "%s"          /* -D / drac-host, if DRAC set */
+                          "%s"          /* -k / kerberos, if KERBEROS set */
+                          "%s"          /* -K / kerberos service, if KERBEROS set */
+                          "%s"          /* -l / tls_support, if QPOP_SSL set */
                           "[-L lock refresh] "
-                          "%s"          /* plain-text passwd stuff, if APOP or SCRAM */
+                          "%s"          /* -p / plain-text passwd stuff, if APOP or SCRAM */
                           "[-R] "
                           "[-s] "
                           "[-S] "
                           "[-t trace-file] "
                           "[-T timeout] "
+                          "[-u] "
+                          "[-U] "
+                          "[-v] "
                           "\n",
-                  argmessage[0], xB, xD, xk, xK, xp );
+                  argmessage[0],
+                  xB,                   /* -B (or not) */
+                  xD,                   /* -D (or not) */
+                  xk,                   /* -k (or not) */
+                  xK,                   /* -K (or not) */
+                  xl,                   /* -l (or not) */
+                  xp );                 /* -p (or not) */
         EXIT ( 1 );
     }
 
+#ifndef   QPOP_SSL
+    if ( p->tls_support != QPOP_TLS_NONE ) {
+        /* This the main ifdef for SSL in the bulk of qpopper. Most of the
+           dependencies are in pop_tls.c */
+         fprintf ( stderr, "TLS/SSL (the -l option) is not supported "
+                           "in this build.\nDetails on adding suport are in "
+                           "the installation instructions\n" );
+         EXIT ( 1 );
+    }
+#endif /* QPOP_SSL */
+
+
     /* set defaults if needed */
 #ifdef    DRAC_AUTH
     if ( p->drac_host == NULL )
@@ -879,13 +1134,9 @@ char    **      argmessage;
 
 #endif /* _DEBUG */
 
-    /*  Create input file stream for TCP/IP communication */
-    if ( ( p->input = fdopen ( sp, "r" ) ) == NULL ) {
-        pop_log ( p, POP_PRIORITY, HERE,
-                  "Unable to open communication stream for input: %s (%d)",
-                  STRERROR(errno), errno );
-        EXIT ( 1 );
-    }
+    /* Create input file stream for TCP/IP communication */
+    /* Just use standard input */
+    p->input_fd = 0;
 
 #ifdef _DEBUG
     sp = 1;
