@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  */
@@ -10,7 +10,7 @@
  * specifies the terms and conditions for redistribution.
  */
 
-#include <config.h>
+#include "config.h"
 #include <stdio.h>
 #include <sys/types.h>
 #include <ctype.h>
@@ -26,6 +26,6 @@ char    *   buf;
     char        *   mp;
 
     for ( mp = buf; *mp != '\0'; mp++ )
-        if ( isupper ( *mp ) )
+        if ( isupper ( (int) *mp ) )
             *mp = (char) tolower ( (int)*mp );
 }
