@@ -1,20 +1,19 @@
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated.  All rights reserved.
- * The license.txt file specifies the terms and conditions for
- * use, modification, and redistribution.
- *
  * Copyright (c) 1989 Regents of the University of California.
  * All rights reserved.  The Berkeley software License Agreement
  * specifies the terms and conditions for redistribution.
  */
 
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
  * Revisions: 
  *
+ * 10/13/00  [rcg]
+ *           - Fitted LGL's changes for TLS.
+ *
  * 04/24/00  [rcg]
  *           - Remove STDC_HEADERS stuff.
  *
@@ -37,21 +36,20 @@
 
 
 
+#include "config.h"
 
 #include <sys/types.h>
 #include <stdio.h>
+#include <stdlib.h>
+#include <stdarg.h>
 #include <string.h>
 
-#include "config.h"
-
 #if HAVE_STRINGS_H
 #  include <strings.h>
 #endif
 
-#include <stdlib.h>
-#include <stdarg.h>
-
 #include "popper.h"
+#include "snprintf.h"
 
 /* 
  *  msg:    Send a formatted line to the POP client
@@ -59,7 +57,7 @@
 
 int
 pop_msg ( POP         *p,         /* it all comes back to this */
-          pop_result   stat,      /* success or failure message? */
+          pop_result   status,    /* success or failure message? */
           const char  *fn,        /* file name whence called */
           size_t       ln,        /* line number whence called */
           const char  *format,    /* format string for message */
@@ -84,7 +82,7 @@ pop_msg ( POP         *p,         /* it all comes back to this */
     /*  
      * Format the POP status code at the beginning of the message 
      */
-    if ( stat == POP_SUCCESS )
+    if ( status == POP_SUCCESS )
         iChunk = Qsprintf ( mp, "%s ", POP_OK );
     else
         iChunk = Qsprintf ( mp, "%s ", POP_ERR );
@@ -105,7 +103,7 @@ pop_msg ( POP         *p,         /* it all comes back to this */
             /* 
              * We blew out the format buffer.
              */
-            pop_log ( p, LOG_NOTICE, fn, ln, 
+            pop_log ( p, POP_NOTICE, fn, ln, 
                       "Buffer size exceeded formatting message: %.4s %s",
                       message, format );
         }
@@ -116,13 +114,13 @@ pop_msg ( POP         *p,         /* it all comes back to this */
     /*  
      * Log the message if debugging is turned on 
      */
-    if ( DEBUGGING && p->debug && stat == POP_SUCCESS )
+    if ( DEBUGGING && p->debug && status == POP_SUCCESS )
         pop_log ( p, POP_DEBUG, fn, ln, "%s", message );
 
     /*  
      * Log the message if a failure occurred 
      */
-    if ( stat != POP_SUCCESS ) {
+    if ( status != POP_SUCCESS ) {
 #ifdef _DEBUG
         pop_log ( p, POP_NOTICE, fn, ln, "%s at localhost: %s", 
                   ( p->user ? p->user : "(null)" ), message );
@@ -141,11 +139,11 @@ pop_msg ( POP         *p,         /* it all comes back to this */
     /*  
      * Send the message to the client 
      */
-    fputs  ( message, p->output );
-    fflush ( p->output );
+    pop_write  ( p, message, strlen(message) );
+    pop_write_flush ( p );
 
     /* 
      * Return the success or failure code passed in to us 
      */
-    return ( stat );
+    return ( status );
 }
