@@ -1,10 +1,30 @@
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
  * Revisions:
  *
+ * 01/31/01  [rcg]
+ *         -  No longer hiding messages deleted in prior aborted session
+ *           when '--disable-update-abort' set.
+ *
+ * 01/30/01  [rcg]
+ *         - Qmaillock now takes bNo_atomic_open and pSpool_dir parameters
+ *           instead of using compile-time constants.
+ *
+ * 01/15/01  [rg]
+ *         - Now using run-time options bKeep_temp_drop, bDo_timing,
+ *           bUpdate_on_abort, bAuto_delete, bUpdate_status_hdrs.
+ *
+ * 10/02/00  [rg]
+ *         - Using rename(2) instead of copying if p->fast_update set.
+ *
+ * 09/21/00  [rg]
+ *         - Added calls on cache_unlink and cache_write.
+ *         - Updating mlp as messages updated (for cache).
+ *         - Updating p->spool_end and p->drop_size (for cache).
+ *
  * 08/16/00  [rg]
  *         - Added error string to standard_error.
  *
@@ -111,18 +131,20 @@ static char standard_error[] =
 void
 unlink_temp_drop ( POP *p, char *filename, const char *fn, size_t ln )
 {
-#ifdef  KEEP_TEMP_DROP
-    DEBUG_LOG3 ( p, "KEEP_TEMP_DROP set [%s:%lu]; temp drop (%s) not unlinked",
-                 fn, ln, filename );
-#else   /* not KEEP_TEMP_DROP */
-    /*
-     * Added code in pop_dropcopy.c makes unlink ok now.
-     * s-dorner@uiuc.edu, 12/91 
-     */
-    unlink ( filename );
-    DEBUG_LOG3 ( p, "Unlinked [%s:%lu] temp drop (%s)", 
-                 fn, (unsigned long) ln, filename );
-#endif  /* KEEP_TEMP_DROP */
+    if ( p->bKeep_temp_drop ) {
+        DEBUG_LOG3 ( p, "bKeep_temp_drop set [%s:%lu]; temp drop (%s) "
+                        "not unlinked",
+                     fn, (unsigned long) ln, filename );
+    }
+    else {
+        /*
+         * Added code in pop_dropcopy.c makes unlink ok now.
+         * s-dorner@uiuc.edu, 12/91 
+         */
+        unlink ( filename );
+        DEBUG_LOG3 ( p, "Unlinked [%s:%lu] temp drop (%s)", 
+                     fn, (unsigned long) ln, filename );
+    }
 }
 
 
@@ -150,6 +172,58 @@ truncate_file ( POP *p, int fd, const char *id, const char *fn, size_t ln )
 
 
 
+/*---------------------------------------------------------------------
+  Flow:
+  =====
+
+Normal mode:
+    1.  If all msgs deleted, truncate, unlink, & close temp drop, and exit.
+    2.  Copy any newly-arrived msgs from spool to end of temp drop.
+    3.  truncate spool (at offset 0).
+    4.  Copy (& update state/uidl) non-deleted msgs from temp drop to spool.
+    5.  Copy new msgs (copied to temp drop in 2) from temp drop to spool.
+    6.  Truncate temp drop (at offset 0).
+    7.  Unlink temp drop.
+    8.  Exit.
+
+
+Server mode:
+    1.  If spool not dirty, unlink temp drop and exit.
+    2.  If all msgs deleted and no new mail arrived, truncate spool & unlink
+        temp drop, and exit.
+    3.  Switch md and mfd from spool to temp drop.
+    4.  Copy (& update state/uidl) non-deleted msgs from spool to temp drop.
+    5.  Copy any newly-arrived msgs from spool to end of temp drop.
+    6.  Truncate spool (at offset 0).
+    7.  If p->fast_update, try to rename temp drop to spool.
+    8.  If p->fast_update not set, or rename failed, copy temp drop to spool.
+    9.  Truncate temp drop (at offset 0).
+   10.  Unlink temp drop.
+   11.  Exit.
+
+
+Restore (non-update):
+    1.  If server mode, unlink temp drop and exit.
+    2.  Copy any newly-arrived msgs from spool to end of temp drop.
+    3.  Truncate (at offset 0).
+    4.  Copy temp drop to spool.
+    5.  Truncate temp drop (at offset 0).
+    6.  Unlink temp drop.
+    7.  Exit.
+
+
+File descriptors:
+-----------------
+md:     Normal mode:    File descriptor for spool.
+        Server mode:    Changes to file descriptor for temp drop.
+mdf:    Normal mode:    FILE * (stream) for spool.
+        Server mode:    Changes to FILE * for temp drop.
+
+
+  ---------------------------------------------------------------------*/
+
+
+
 
 /* 
  *  updt:   Apply changes to a user's POP maildrop
@@ -170,61 +244,55 @@ POP     *   p;
                                                         counter */
     register int            status_written;         /*  Status header field 
                                                         written */
-    int                     nchar;                  /* Bytes read/written */
+    int                     nchar  = 0;             /*  Bytes read/written */
 
-    long                    offset = 0;             /* New mail offset */
+    long                    offset = 0;             /*  New mail offset */
 
-    int                     save_errno = 0;         /* Save the error value we
-                                                       are trying to print. */
-    int                     rslt = 0;               /* temp use when calling */
+    int                     save_errno = 0;         /*  Save the error value we
+                                                        are trying to print. */
+    int                     rslt = 0;               /*  temp use when calling */
     struct stat             mybuf;                  /*  For fstat() */
     time_t                  time_locked = 0;
-    int                     retrycnt    = 0;        /* for patient locking */
-
-#ifdef DO_TIMING
-    time_t                  my_timer = time(0);
-#endif /* DO_TIMING */
+    int                     retrycnt    = 0;        /*  For patient locking */
+    long                    new_end     = 0;        /*  New end of spool */
+    time_t                  my_timer    = time(0);  /*  For timing */
 
 
-#ifdef AUTO_DELETE
-#  ifdef NOUPDATEONABORT
-    /* 
-       AUTO_DELETE causes messages which have been RETRd by the client to be
-       unconditionally and automatically marked deleted.  This prevents clients
-       from leaving mail on the server.  
+    if ( p->bAuto_delete && p->bUpdate_on_abort == FALSE )
+    {
+        /* 
+           bAuto_delete causes messages which have been RETRd by the client to be
+           unconditionally and automatically marked deleted.  This prevents clients
+           from leaving mail on the server.  
 
-       CAUTION: Be sure users are informed of this before enabling it, to 
-       prevent lost mail.
+           CAUTION: Be sure users are informed of this before enabling it, to 
+           prevent lost mail.
 
-       Note: NOUPDATEONABORT must also be defined; otherwise users may lose mail
-       if they try and RETR it but the connection closes for any reason, the 
-       client aborts, etc.
-    */
-    if ( p->msgs_deleted < p->msg_count )
-    { /* there are msgs available for deletion */
-        DEBUG_LOG0 ( p, "Auto-deleting messages which have been RETRd..." );
-        for ( msg_num = 0; msg_num < p->msg_count; ++msg_num )
-        { /* msg loop */
-            mp = &p->mlp [ msg_num ];
-            if ( mp->retr_flag && !mp->del_flag )
-            {
-                p->msgs_deleted  ++;
-                p->bytes_deleted += mp->length;
-                p->dirty          = TRUE;
-                mp->del_flag      = TRUE;
-                DEBUG_LOG1 ( p, ">>> msg %i automatically flagged as DELETED <<<", 
-                             msg_num );
-            }
-    
-        } /* msg loop */
-    } /* there are msgs available for deletion */
-#  endif /* NOUPDATEONABORT */
-#endif /* AUTO_DELETE */
+           Note: bUpdate_on_abort must be FALSE; otherwise users may lose mail
+           if they try and RETR it but the connection closes for any reason, the 
+           client aborts, etc.
+        */
+        if ( p->msgs_deleted < p->msg_count )
+        { /* there are msgs available for deletion */
+            DEBUG_LOG0 ( p, "Auto-deleting messages which have been RETRd..." );
+            for ( msg_num = 0; msg_num < p->msg_count; ++msg_num )
+            { /* msg loop */
+                mp = &p->mlp [ msg_num ];
+                if ( mp->retr_flag && !mp->del_flag )
+                {
+                    delete_msg ( p, mp );
+                    DEBUG_LOG1 ( p, ">>> msg %i automatically flagged as DELETED <<<", 
+                                 msg_num );
+                }
+        
+            } /* msg loop */
+        } /* there are msgs available for deletion */
+    } /* bAuto_delete && bUpdate_on_abort == FALSE */
 
     DEBUG_LOG0 ( p, "Performing maildrop update..." );
     DEBUG_LOG0 ( p, "Checking to see if all messages were deleted" );
 
-    if ( p->stats ) {
+    if ( p->bStats ) {
         pop_log ( p, POP_PRIORITY, HERE, "Stats: %s %d %ld %d %ld %s %s",
                   p->user, p->msgs_deleted, p->bytes_deleted,
                   p->msg_count - p->msgs_deleted,
@@ -232,21 +300,31 @@ POP     *   p;
                   p->client, p->ipaddr );
     }
 
-    if ( p->server_mode ) {
-        if ( !p->dirty ) {
-            unlink_temp_drop ( p, p->temp_drop, HERE );
-            DEBUG_LOG0 ( p, "Running in server mode; spool not "
-                            "changed; deleting temp drop" );
-#ifdef DO_TIMING
-            p->clean_time = time(0) - my_timer;
-#endif /* DO_TIMING */
-
-            return ( POP_SUCCESS );
-        }
-    }
-    if ( !p->server_mode && p->msgs_deleted == p->msg_count ) {
+    if ( p->server_mode && p->dirty == FALSE ) {
         /*
-         * In a !server_mode the server process would be the sole user of
+         * Write out cache file
+         */
+        rslt = cache_write ( p, fileno(p->drop), p->mlp );
+        if ( rslt == POP_FAILURE )
+            cache_unlink ( p );
+
+        unlink_temp_drop ( p, p->temp_drop, HERE );
+        DEBUG_LOG0 ( p, "Running in server mode; spool not "
+                        "changed; deleting temp drop" );
+    if ( p->bDo_timing )
+        p->clean_time = time(0) - my_timer;
+
+        return ( POP_SUCCESS );
+    } /* server mode and spool not dirty */
+
+    /*
+     * Delete the cache file, just to cover all paths.
+     */
+    cache_unlink ( p );
+
+    if ( p->server_mode == FALSE && p->msgs_deleted == p->msg_count ) {
+        /*
+         * In non-server-mode the Qpopper process is the sole user of
          * .user.pop so truncate and unlink is not a problem.
          *
          * Truncate before close, to avoid race condition.  
@@ -257,18 +335,23 @@ POP     *   p;
         unlink_temp_drop ( p,  p->temp_drop, HERE );
         fclose ( p->drop );
 
-#ifdef DO_TIMING
+    if ( p->bDo_timing )
         p->clean_time = time(0) - my_timer;
-#endif /* DO_TIMING */
 
         return ( POP_SUCCESS );
-    }
+    } /* non-server mode and all msgs deleted */
 
     /* 
      * Always use Qmaillock()
      */
 
-    rslt = Qmaillock ( p->user, 4, p->trace, HERE, p->debug );
+    rslt = Qmaillock ( p->drop_name,
+                       4,
+                       p->bNo_atomic_open,
+                       p->pCfg_spool_dir,
+                       p->trace,
+                       HERE,
+                       p->debug );
     if ( rslt != 0 )
         return ( pop_msg ( p, POP_FAILURE, HERE, 
                            "maillock error '%s' (%d) on '%s': %s (%d)", 
@@ -276,7 +359,7 @@ POP     *   p;
                            STRERROR(errno), errno ) );
     time_locked = time(0);
     
-    if ( !p->server_mode ) {
+    if ( p->server_mode == FALSE ) {
         DEBUG_LOG1 ( p, "(Normal mode) opening spool \"%s\"", 
                      p->drop_name );
         /*  
@@ -291,7 +374,7 @@ POP     *   p;
         
         DEBUG_LOG2 ( p, "(normal mode) mfd=%d: %s", mfd, p->drop_name );
     } 
-    else {
+    else { /* server mode */
         mfd = fileno ( p->drop );
         DEBUG_LOG2 ( p, "(Server mode) accessing spool; mfd=%d: %s", 
                      mfd, p->drop_name );
@@ -310,7 +393,7 @@ POP     *   p;
                          STRERROR(errno), errno );
     }
 
-    if ( !p->server_mode )  {
+    if ( p->server_mode == FALSE )  {
         /* 
          * Go to the right places 
          */
@@ -357,7 +440,8 @@ POP     *   p;
         rewind ( p->drop );
         lseek  ( fileno ( p->drop ), (OFF_T)0, L_SET );
 
-        /*  Transfer messages not flagged for deletion from the temporary 
+        /*
+         *  Transfer messages not flagged for deletion from the temporary 
          *  maildrop to the new maildrop 
          */
         DEBUG_LOG2 ( p, "Creating new maildrop \"%s\" from \"%s\"",
@@ -365,45 +449,54 @@ POP     *   p;
 
     } /* not server mode */
     else { /* SERVER_MODE */
-        fstat ( fileno ( p->drop ), &mybuf );
+        fstat ( mfd, &mybuf );
         if ( p->msgs_deleted == p->msg_count &&
              mybuf.st_size   == p->spool_end ) {
             /* 
              * Truncate before close, to avoid race condition.  
              */
-            truncate_file ( p, fileno ( p->drop ), "spool", HERE );
+            truncate_file ( p, mfd, "spool", HERE );
             DEBUG_LOG1 ( p, "Server mode; all msgs deleted and spool size "
                             "unchanged; truncated spool (%d)",
-                            fileno ( p->drop) );
+                            mfd );
             unlink_temp_drop ( p,  p->temp_drop, HERE );
+            cache_unlink ( p );
             flock  ( mfd, LOCK_UN );
             fclose ( p->drop );
             fclose ( p->hold );
             Qmailunlock ( HERE );
 
-#ifdef DO_TIMING
-            p->clean_time = time(0) - my_timer;
-#endif /* DO_TIMING */
+            if ( p->bDo_timing )
+                p->clean_time = time(0) - my_timer;
 
             return ( POP_SUCCESS );
         } /* all msgs deleted; no new ones arrived */
 
         md  = p->hold;   /* Really the temp drop */
         mfd = fileno ( md );
-        DEBUG_LOG1 ( p, "server mode: set md to p->hold; "
-                        "mfd = temp drop (%d)", mfd );
+        DEBUG_LOG1 ( p, "server mode: set md/mfd to temp drop (p->hold): %d",
+                     mfd );
     } /* server mode */
 
-    if ( !p->server_mode || ( p->msgs_deleted != p->msg_count ) ) {
-        DEBUG_LOG5 ( p, "Server mode=%i; %i out of %i msgs deleted; "
-                        "copying msgs from temp drop (%d) to spool (%d)", 
+    if ( p->server_mode == FALSE
+         || ( p->msgs_deleted != p->msg_count ) ) {
+        DEBUG_LOG7 ( p, "Server mode=%i; %i out of %i msgs deleted; "
+                        "copying msgs from %s (%d) to %s (%d)", 
                      p->server_mode, p->msgs_deleted, p->msg_count,
-                     fileno(p->drop), fileno(md) );
+                     p->server_mode ? "spool" : "temp drop",
+                     fileno(p->drop),
+                     p->server_mode ? "temp drop" : "spool",
+                     mfd );
+
+        p->drop_size = 0; /* Recalculate */
 
         for ( msg_num = 0; msg_num < p->msg_count; ++msg_num ) {
 
-            int inheader   = 1;
-            int body_lines = 0;
+            int  inheader   = 1;
+            int  body_lines = 0;
+            int  lines      = 0;
+            long length     = 0;
+            long offset     = 0;
 
             /*  
              * Get a pointer to the message information list 
@@ -427,11 +520,12 @@ POP     *   p;
             }
 
             fseek ( p->drop, mp->offset, SEEK_SET );
-
-            DEBUG_LOG1 ( p, "Copying message %d.", mp->number );
+            
+            offset = ftell ( md );
 
             /* 
              * Put the From line separator 
+             * (Does not count in line count or length)
              */
             fgets ( buffer, MAXMSGLINELEN, p->drop );
             if ( fputs ( buffer, md ) == EOF )
@@ -440,6 +534,8 @@ POP     *   p;
             sprintf ( buffer, "X-UIDL: %s", mp->uidl_str );
             if ( fputs ( buffer, md ) == EOF )
                 break;
+            length += strlen ( buffer ) + 1; /* for CRLF */
+            lines  ++;
 
             for ( status_written = 0, inheader = 1;
                   fgets ( buffer, MAXMSGLINELEN, p->drop );
@@ -451,15 +547,14 @@ POP     *   p;
                      * A blank line signals the end of the header. 
                      */
                     if ( *buffer == '\n' ) {
-#ifndef NO_STATUS
-                        if ( status_written == 0 ) {
+                        if ( p->bUpdate_status_hdrs 
+                             && status_written == FALSE ) {
                             if ( mp->retr_flag ) {
                                 sprintf ( buffer, "Status: RO\n\n" );
                             } else {
                                 sprintf ( buffer, "Status: U\n\n" );
                             }
-                        }
-#endif /* NO_STATUS */
+                        } /* bUpdate_status_hdrs && status_written == FALSE */
 
                         inheader       = 0;
                         body_lines     = 1;
@@ -468,6 +563,14 @@ POP     *   p;
                     } else if ( !strncasecmp ( buffer, "X-UIDL:", 7 ) ) {
                         continue;       /* Skip all existing UIDL lines */
 
+                    /*
+                     * Content-Length header counts as a line, but not 
+                     * as length.
+                     */
+                    } else if ( CONTENT_LENGTH && 
+                                !strncmp ( buffer, "Content-Length:", 15 ) ) {
+                        length -= strlen ( buffer ) + 1; /* for CRLF */
+
                     } else if ( !strncasecmp ( buffer, "Status:", 7 ) ) {
 
                         /*  
@@ -482,12 +585,18 @@ POP     *   p;
                      */
                     if ( fputs ( buffer, md ) == EOF )
                         break;
+                    length += strlen ( buffer ) + 1; /* for CRLF */
+                    lines  ++;
 
                     } else { /* Body */ 
-                        if ( ++body_lines > mp->body_lines )
+                        if ( ++body_lines > mp->body_lines ) {
+                            body_lines--;
                             break;
+                        }
                         if ( fputs ( buffer, md ) == EOF )
                             break;
+                        length += strlen ( buffer ) + 1; /* for CRLF */
+                        lines  ++;
                     }
             }
 
@@ -498,7 +607,19 @@ POP     *   p;
             if ( p->mmdf_separator ) {
                 fputs ( p->mmdf_separator, md );
             }
-        }
+            
+            DEBUG_LOG9 ( p, "Copied message %d: offset %ld (was %ld); "
+                            "length %ld (was %ld); lines %d (was %d); "
+                            "body_lines %d (was %d)",
+                         mp->number, offset, mp->offset, length, mp->length,
+                         lines,  mp->lines,  body_lines, mp->body_lines );
+
+            mp->offset      = offset;
+            mp->length      = length;
+            mp->lines       = lines;
+            mp->body_lines  = mp->body_lines;
+            p->drop_size   += length;
+        } /* msg loop */
 
     /* flush and check for errors now!  The new mail will be writen
      * without stdio,  since we need not separate messages 
@@ -513,8 +634,8 @@ POP     *   p;
             fclose ( p->drop );
             if ( save_errno == EDQUOT )
                 return pop_msg ( p, POP_FAILURE, HERE,
-                                 "Overquota copying messages to Mailspool."
-                                 " Temp drop unchanged (%d)",
+                                 "Overquota copying messages to spool."
+                                 "  Temp drop unchanged (%d)",
                                  save_errno );
             else
                 return pop_msg ( p, POP_FAILURE, HERE, 
@@ -541,6 +662,13 @@ POP     *   p;
                                  standard_error, STRERROR ( save_error ),
                                  save_error );
         }
+        
+        /*
+         * Update the spool EOF (we may have increased it by writing X-UIDL
+         * and/or Status headers) and we want it to be correct in the cache
+         * file.
+         */
+        new_end = ftell ( md );
 
     } /* p->msgs_deleted != p->msg_count */
 
@@ -553,10 +681,10 @@ POP     *   p;
             /* 
              * Go to the right places 
              */
-            lseek ( fileno ( p->drop ), (OFF_T)p->spool_end, L_SET ); 
+            lseek ( fileno ( p->drop ), (OFF_T) p->spool_end, L_SET ); 
             
             DEBUG_LOG1 ( p, "%lu octets were added to spool during session",
-                         mybuf.st_size - p->spool_end );
+                         (long unsigned) mybuf.st_size - p->spool_end );
 
             Qtouchlock ( HERE );
             
@@ -601,31 +729,89 @@ POP     *   p;
 
         Qtouchlock ( HERE );
 
-        while ( ( nchar = read ( mfd, buffer, BUFSIZE ) ) > 0 )
-            if ( nchar != write ( fileno ( p->drop ), buffer, nchar ) ) {
-                nchar = -1;
-                break ;
+        /*
+         * Now the spool is empty and the temp drop contains the updated
+         * complete spool.  If p->fast_update is set, try using rename(2)
+         * to move the temp drop to the spool.  If this fails (because 
+         * they are on different volumes, for example), or if the option
+         * isn't set, copy the temp drop to the spool, then truncate and
+         * unlink the temp spool.
+         */
+        if ( p->bFast_update ) {
+            nchar = rename ( p->temp_drop, p->drop_name );
+            if ( nchar == 0 ) {
+                DEBUG_LOG2 ( p, "Moved %s to %s",
+                             p->temp_drop, p->drop_name );
+                close  ( mfd );
+                fclose ( md );
+                mfd = open ( p->temp_drop, O_RDWR | O_CREAT, 0600 );
+                if ( mfd >= 0 ) {
+                    DEBUG_LOG1 ( p, "Reopened mfd as temp drop (%d)", mfd );
+                    md = fdopen ( mfd, "r+" );
+                    if ( md == NULL )
+                        pop_log ( p, POP_NOTICE, HERE,
+                                  "Unable to reopen md from mfd (%d): %s (%d)",
+                                  mfd, STRERROR(errno), errno );
+                }
+                else
+                    pop_log ( p, POP_NOTICE, HERE,
+                              "Unable to open temporary maildrop "
+                              "'%s': %s (%d)",
+                              p->temp_drop, STRERROR(errno), errno );
             }
+            else {
+                pop_log ( p, POP_NOTICE, HERE,
+                          "Unable to move %s to %s: %s (%d)",
+                          p->temp_drop, p->drop_name,
+                          STRERROR(errno), errno );
+            }
+        } /* p->fast_update */
+        else
+            DEBUG_LOG0 ( p, "(p->fast_update not set)" );
 
-        DEBUG_LOG3 ( p, "%scopying temp drop (%d) to spool (%d)",
-                     nchar == 0 ? "" : "ERROR ",
-                     mfd, fileno(p->drop) );
+        if ( p->bFast_update == FALSE || nchar == -1 ) {
+            while ( ( nchar = read ( mfd, buffer, BUFSIZE ) ) > 0 )
+                if ( nchar != write ( fileno ( p->drop ), buffer, nchar ) ) {
+                    nchar = -1;
+                    break ;
+                }
 
-        if ( nchar != 0 ) {
-            truncate_file ( p, fileno(p->drop), "spool", HERE );
-            fclose ( p->drop );
-            Qmailunlock ( HERE );
-            fclose ( md );
-            if ( errno == EDQUOT ) {
-                return pop_msg ( p, POP_FAILURE, HERE,
-                                 "Overquota copying messages back to mailspool (%d)", 
-                                 errno );
-            } else
-                return pop_msg ( p, POP_FAILURE, HERE,
-                                 "Error appending messages from temporary drop "
-                                 "to mailspool: %s (%d)",
-                                 STRERROR(errno), errno );
-        } /* failed to write */
+            DEBUG_LOG3 ( p, "%scopying temp drop (%d) to spool (%d)",
+                         nchar == 0 ? "" : "ERROR ",
+                         mfd, fileno(p->drop) );
+
+            if ( nchar != 0 ) {
+                truncate_file ( p, fileno(p->drop), "spool", HERE );
+                fclose ( p->drop );
+                Qmailunlock ( HERE );
+                fclose ( md );
+                if ( errno == EDQUOT ) {
+                    return pop_msg ( p, POP_FAILURE, HERE,
+                                     "Overquota copying messages back to "
+                                     "mailspool (%d)",
+                                     errno );
+                } else
+                    return pop_msg ( p, POP_FAILURE, HERE,
+                                     "Error copying messages from temp "
+                                     "drop back to mailspool: %s (%d)",
+                                     STRERROR(errno), errno );
+            } /* failed to write */
+        } /* p->fast_update == FALSE || nchar == -1 */
+
+        /*
+         * Update the spool end to messages we wrote (not counting any new
+         * ones that arrived during the session) for the cache file.
+         */
+        DEBUG_LOG2 ( p, "Updated p->spool_end from %ld to %ld (for cache)",
+                     p->spool_end, new_end );
+        p->spool_end = new_end;
+
+        /*
+         * Write out cache file
+         */
+        rslt = cache_write ( p, fileno(p->drop), p->mlp );
+        if ( rslt == POP_FAILURE )
+            cache_unlink ( p );
 
         truncate_file ( p, mfd, "temp drop", HERE );
         unlink_temp_drop ( p, p->temp_drop, HERE );
@@ -634,6 +820,9 @@ POP     *   p;
         Qmailunlock ( HERE );
     } /* server mode */
     else {
+        DEBUG_LOG2 ( p, "non-server mode; copying any new msgs back from"
+                        "temp drop (%d) to spool (%d)",
+                     fileno(p->drop), mfd );
         /* 
          * Go to start of new mail if any 
          */
@@ -678,21 +867,19 @@ POP     *   p;
         fclose ( p->drop );
         Qmailunlock ( HERE );
 
-    }
+    } /* non-server mode */
 
-#ifdef DO_TIMING
-    p->clean_time = time(0) - my_timer;
-#endif /* DO_TIMING */
+    if ( p->bDo_timing )
+        p->clean_time = time(0) - my_timer;
 
     return ( pop_quit ( p ) );
 }
 
 
 
-int pop_restore ( p )
-POP *p;
+int pop_restore ( POP *p )
 {
-  FILE    *md           = NULL;
+  FILE   *md            = NULL;
   int     mfd           = -1;
   char    buffer [ BUFSIZE ];
   int     nchar         = 0;
@@ -700,26 +887,52 @@ POP *p;
   int     save_errno    = 0;
   int     rslt          = 0;
   int     retrycnt      = 0;
+  time_t  my_timer      = time(0);
 
-#ifdef DO_TIMING
-  time_t  my_timer = time(0);
-#endif /* DO_TIMING */
+    DEBUG_LOG0 ( p, "Performing maildrop restoration..." );
 
-    DEBUG_LOG0 ( p, "Performing maildrop restoration..." ) ;
+    /*
+     * Undelete any deleted messages so cache is correct.
+     */
+    if ( p->msgs_deleted > 0 ) {
+        MsgInfoList    *mp     = NULL;
+        int             msgx   = 0;
 
+        for ( msgx = 0; msgx < p->msg_count; ++msgx ) {
+                mp = &p->mlp [ msgx ];
+                if ( mp->del_flag )
+                    undelete_msg ( p, mp );
+        } /* msg loop */
+    } /* msgs_deleted > 0 */
+
+    /*
+     * In server mode we don't need to do much...
+     */
     if ( p->server_mode ) {
         unlink_temp_drop ( p,  p->temp_drop, HERE );
 
-#ifdef DO_TIMING
-        p->clean_time = time(0) - my_timer;
-#endif /* DO_TIMING */
+        if ( p->bDo_timing )
+            p->clean_time = time(0) - my_timer;
+
+        /*
+         * Write out cache file
+         */
+        rslt = cache_write ( p, fileno(p->drop), p->mlp );
+        if ( rslt == POP_FAILURE )
+            cache_unlink ( p );
 
         return ( POP_SUCCESS );
-    }
+    } /* server_mode */
 
     DEBUG_LOG1 ( p, "Opening mail drop \"%s\"", p->drop_name );
 
-    rslt = Qmaillock ( p->user, 4, p->trace, HERE, p->debug );
+    rslt = Qmaillock ( p->drop_name,
+                       4,
+                       p->bNo_atomic_open,
+                       p->pCfg_spool_dir,
+                       p->trace,
+                       HERE,
+                       p->debug );
     if ( rslt != 0 )
         return ( pop_msg ( p, POP_FAILURE, HERE, 
                            "maillock error '%s' (%d) on '%s': %s (%d)", 
@@ -745,7 +958,8 @@ POP *p;
     fseek ( p->drop, 0, SEEK_END );
     offset = ftell ( p->drop );
 
-    /* Append any messages that may have arrived during the session 
+    /*
+     * Append any messages that may have arrived during the session 
      * to the temporary mail drop 
      */
     while ( ( nchar = read ( mfd, buffer, BUFSIZE ) ) > 0 )
@@ -758,74 +972,76 @@ POP *p;
                  nchar == 0 ? "" : "ERROR ",
                  mfd, fileno(p->drop) );
 
-        if ( nchar != 0) {
-            save_errno = errno;
-            fclose ( md );
-            ftruncate ( fileno(p->drop), (OFF_T)offset );
-            DEBUG_LOG2 ( p, "ERROR; truncated temp drop (%d) to %lu",
-                         fileno(p->drop), offset );
-            fclose ( p->drop );
-            Qmailunlock ( HERE );
+    if ( nchar != 0) {
+        save_errno = errno;
+        fclose ( md );
+        ftruncate ( fileno(p->drop), (OFF_T)offset );
+        DEBUG_LOG2 ( p, "ERROR; truncated temp drop (%d) to %lu",
+                     fileno(p->drop), offset );
+        fclose ( p->drop );
+        Qmailunlock ( HERE );
 
-            if ( save_errno == EDQUOT ) {
-                return pop_msg ( p, POP_FAILURE, HERE, 
-                                 "Overquota: appending messages "
-                                 "to temporary drop (%d)",
-                                 save_errno );
-            }
-            else
-                return pop_msg ( p, POP_FAILURE, HERE,
-                                 "Error appending messages from mailspool "
-                                 "to temporary drop: %s (%d)",
-                                 STRERROR(save_errno), save_errno );
-       }
+        if ( save_errno == EDQUOT ) {
+            return pop_msg ( p, POP_FAILURE, HERE, 
+                             "Overquota: appending messages "
+                             "to temporary drop (%d)",
+                             save_errno );
+        }
+        else
+            return pop_msg ( p, POP_FAILURE, HERE,
+                             "Error appending messages from mailspool "
+                             "to temporary drop: %s (%d)",
+                             STRERROR(save_errno), save_errno );
+    }
 
-       fflush ( md );
-       rewind ( md );
-       truncate_file ( p, mfd, "spool", HERE );
-       lseek ( mfd, (OFF_T)0, L_SET );
+    fflush ( md );
+    rewind ( md );
+    truncate_file ( p, mfd, "spool", HERE );
+    lseek ( mfd, (OFF_T)0, L_SET );
 
-       rewind ( p->drop );
-       lseek ( fileno(p->drop), (OFF_T)0, L_SET );
-       
-       while ( ( nchar = read ( fileno(p->drop), buffer, BUFSIZE ) ) > 0 )
-            if ( nchar != write ( mfd, buffer, nchar ) ) {
-                nchar = -1;
-                break;
-            }
+    rewind ( p->drop );
+    lseek ( fileno(p->drop), (OFF_T)0, L_SET );
+    
+    while ( ( nchar = read ( fileno(p->drop), buffer, BUFSIZE ) ) > 0 )
+         if ( nchar != write ( mfd, buffer, nchar ) ) {
+             nchar = -1;
+             break;
+         }
 
-        DEBUG_LOG3 ( p, "%scopying from temp drop (%d) to spool (%d)",
-                 nchar == 0 ? "" : "ERROR ",
-                 fileno(p->drop), mfd );
+     DEBUG_LOG3 ( p, "%scopying from temp drop (%d) to spool (%d)",
+              nchar == 0 ? "" : "ERROR ",
+              fileno(p->drop), mfd );
 
-       if ( nchar != 0 ) {
-           save_errno = errno;
-           truncate_file ( p, mfd, "spool", HERE );
-           fclose ( md );
-           fclose ( p->drop );
-           Qmailunlock ( HERE );
+    if ( nchar != 0 ) {
+        save_errno = errno;
+        truncate_file ( p, mfd, "spool", HERE );
+        fclose ( md );
+        fclose ( p->drop );
+        Qmailunlock ( HERE );
 
-           if ( save_errno == EDQUOT )
-               return pop_msg ( p, POP_FAILURE, HERE,
-                                "Overquota copying messages to Mailspool. "
-                                "Temp drop unchanged (%d)",
-                                save_errno );
-           else
-               return pop_msg ( p, POP_FAILURE, HERE, standard_error,
-                                STRERROR ( errno ), errno );
-       }
+        if ( save_errno == EDQUOT )
+            return pop_msg ( p, POP_FAILURE, HERE,
+                             "Overquota copying messages to Mailspool. "
+                             "Temp drop unchanged (%d)",
+                             save_errno );
+        else
+            return pop_msg ( p, POP_FAILURE, HERE, standard_error,
+                             STRERROR ( errno ), errno );
+    }
 
-       fclose ( md );
-       truncate_file ( p, fileno(p->drop), "temp drop", HERE );
-       unlink_temp_drop ( p,  p->temp_drop, HERE );
-       fclose ( p->drop );
-       Qmailunlock ( HERE );
+    fclose ( md );
 
-#ifdef DO_TIMING
-    p->clean_time = time(0) - my_timer;
-#endif /* DO_TIMING */
+    cache_unlink ( p );
 
-       return pop_quit ( p );
+    truncate_file ( p, fileno(p->drop), "temp drop", HERE );
+    unlink_temp_drop ( p,  p->temp_drop, HERE );
+    fclose ( p->drop );
+    Qmailunlock ( HERE );
+
+    if ( p->bDo_timing )
+        p->clean_time = time(0) - my_timer;
+
+    return pop_quit ( p );
 }
 
 
