@@ -5,28 +5,48 @@
  */
 
 /*
- * Copyright (c) 2000 QUALCOMM Incorporated. All rights reserved.
+ * Copyright (c) 2001 QUALCOMM Incorporated. All rights reserved.
  * See License.txt file for terms and conditions for modification and
  * redistribution.
  *
  * Revisions:
+ *
+ *  04/28/01 [rg]
+ *           - Added fix for XTND XMIT (sent in by Jacques Distler and
+ *    		   others).
+ *
+ *  02/02/01 [rg]
+ *           - Accounted for tgetline now stripping CRLF from input.
+ *           - Now using p->log_facility.
+ *
+ *  10/13/00 [rg]
+ *           - Added LGL's patch accounting for p->input_fd.
+ *           - Unlinking temp file now conditional on _DEBUG instead
+ *             of _DEBUG.
+ *
  *  06/11/00 [rg]
  *           - Applied patch by Clifton Royston to translate network EOL
  *             (CRLF) to local EOL ('\n').
  *           - Now only initial ".\n' is recognized as end-of-msg (instead
  *             of any line which starts with a "." and ends with a ".").
+ *
  *  03/10/00 [rg]
  *           - Additional change suggested by Jacques Distler for NEXT systems.
+ *
  *  02/02/00 [rg]
  *           - Made changes suggested by Jacques Distler for NEXT systems.
+ *
  *  01/14/00 [rg]
  *           - Used new genpath() call to get POP_TMPXMIT path.
+ *
  *  12/17/99 [rg]
  *           - Made sure all functions return a value, to appease some 
  *             compilers.
+ *
  *  06/24/98 [py]
  *           - Added a deprecated switch (-i) to sendmail to ignore 
  *             dots. (Need to find another method.)
+ *
  *  04/04/98 [py]
  *           -Made callstack and xmit_recv.
  *
@@ -90,12 +110,7 @@ POP    *  p;
      * Create a temporary file into which to copy the user's message 
      */
 
-    if ( genpath ( p->user, 
-                   temp_xmit, 
-                   sizeof(temp_xmit),
-                   GNPH_XMT,
-                   (p->debug ? p->trace : NULL),
-                   p->debug ) < 0 )
+    if ( genpath ( p, temp_xmit, sizeof(temp_xmit), GNPH_XMT ) < 0 )
         return ( pop_msg ( p, POP_FAILURE, HERE, 
                            "Unable to get temporary message file name" ) );
 
@@ -130,14 +145,6 @@ char *buffer;
      * Data written to a file should follow UNIX or local EOL format ('\n').
      * sendmail and some MTAs will ignore superfluous CRs, but not all will.
      */
-    int l = strlen(buffer) -1;
-
-    if ( ( l > 0 ) 
-      && ( buffer[l] == '\n' ) && ( buffer[l-1] == '\r' ) ) {
-        /* Eliminate trailing \r from network EOL format (CR-LF). */
-        buffer[l-1] = '\n'; 
-        buffer[l]   = '\0';
-    }
 
     /*  
      * Look for initial period 
@@ -147,22 +154,33 @@ char *buffer;
         /*  
          * Exit on end of message 
          */
-        if ( buffer[1] == '\n' || buffer[1] == '\r' ) {
+        if ( buffer[1] == '\0' ) {
             DEBUG_LOG0 ( p, "Received end of msg" );
             fclose ( tmp );
             p->xmitting = FALSE;
             Pop ( &(p->InProcess) );
             pop_xmit_exec ( p );
         }
+
         /* 
-         * sendmail will not remove escaped .. 
+         * sendmail will not remove escaped dot 
          */
-        else if ( buffer[1] == '.' )
+        else if ( buffer[1] == '.' ) {
             fputs ( &buffer[1], tmp );
-        else    
+            fputc ( '\n', tmp );
+        }
+        else {
             fputs ( buffer, tmp );
-    } else    
+            fputc ( '\n', tmp );
+            DEBUG_LOG4 ( p, "Client sent unescaped leading dot: %s at %s (%s): %.60s",
+            			 p->user, p->client, p->ipaddr, buffer );
+        }
+    } /* buffer[0] == '.' */
+
+    else {
         fputs ( buffer, tmp );
+        fputc ( '\n', tmp );
+    }
     return 0;
 }
 
@@ -180,7 +198,7 @@ POP *p;
 
     PID_T                   id, pid;
     
-    DEBUG_LOG1 ( p, "Forking for \"%s\"",  MAIL_COMMAND );
+    DEBUG_LOG1 ( p, "Forking for \"%s\"",  p->pMail_command );
     DEBUG_LOG1 ( p, "Reading from \"%s\"", temp_xmit );
     pop_log ( p, POP_DEBUG, HERE, "Pop transmit from \"%s\" on \"%s\"", 
               p->user, p->client );
@@ -196,28 +214,29 @@ POP *p;
 #ifdef SYSLOG42
             (void) openlog ( p->myname, 0 );
 #else
-            (void) openlog ( p->myname, LOG_PID, POP_FACILITY );
+            (void) openlog ( p->myname, LOG_PID, p->log_facility );
 #endif
-            pop_log ( p, POP_DEBUG, HERE, "Pop transmit from \"%s\" on \"%s\"", 
+            pop_log ( p, POP_DEBUG, HERE, "Pop transmit from \"%s\" on \"%s\"",
                       p->user, p->client );
-            (void) fclose ( p->input );
-            (void) fclose ( p->output );       
-            (void) close(0);   /* Make sure open return stdin.
-                                * to be used by MAIL_COMMAND
-                                */
+            (void) close  ( p->input_fd  );
+            (void) fclose ( p->output    );
+            (void) close  ( 0            );   /* Make sure open return stdin.
+                                               * to be used by MAIL_COMMAND
+                                               */
             if ( open ( temp_xmit, O_RDONLY, 0 ) < 0 ) {
                 DEBUG_LOG3 ( p, "Unable to open \"%s\": %s (%d)",
                              temp_xmit, STRERROR(errno), errno );
-                (void) _exit(1);
+                (void) _exit ( 1 );
             }
-            (void) execl ( MAIL_COMMAND, "send-mail", "-t", "-i", "-oem", NULLCP );
-            (void) _exit(1);
+            (void) execl ( p->pMail_command, "send-mail", "-t", "-i", "-oem", NULLCP );
+            (void) _exit ( 1 );
 
         case -1:
-            if ( !p->debug ) 
+#ifndef   _DEBUG
                 (void) unlink ( temp_xmit );
+#endif /* _DEBUG */
             return pop_msg ( p, POP_FAILURE, HERE, "Unable to execute \"%s\": %s (%d)",
-                             MAIL_COMMAND, STRERROR(errno), errno );
+                             p->pMail_command, STRERROR(errno), errno );
         default:
 
             DEBUG_LOG1 ( p, "waiting on forked process (%u)",
@@ -227,28 +246,35 @@ POP *p;
 #else
             id = waitpid ( pid, &status, 0 );
 #endif
-            if ( !p->debug ) 
+
+#ifndef   _DEBUG
                 (void) unlink ( temp_xmit );
+#endif /* _DEBUG */
 
 #ifdef NEXT
             if ( !WIFEXITED (*status) )
 #else
             if ( (!WIFEXITED (status) ) || ( WEXITSTATUS (status) != 0) )
 #endif
-                return pop_msg ( p, POP_FAILURE, HERE, "Unable to send message" );
+                return pop_msg ( p, POP_FAILURE, HERE, 
+                                 "Unable to send message" );
 
-            return pop_msg ( p, POP_SUCCESS, HERE, "Message sent successfully" );
+            return pop_msg ( p, POP_SUCCESS, HERE,
+                             "Message sent successfully" );
     }
 }
 
 
 int
-pop_xmit_clean ( p )
-POP *p;
+pop_xmit_clean ( POP *p )
 {
+    (void) p;
     fclose ( tmp );
-    if ( !p->debug )
-        unlink (temp_xmit);
+
+#ifndef _DEBUG
+    unlink ( temp_xmit );
+#endif /* _DEBUG */
+
     return 0;
 }
 
