@@ -14,7 +14,7 @@
  * specifies the terms and conditions for redistribution.
  */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <sys/types.h>
