@@ -5,7 +5,7 @@
  */
  
 /*
- * Copyright (c) 2000 Qualcomm Incorporated.  All rights reserved.
+ * Copyright (c) 2001 Qualcomm Incorporated.  All rights reserved.
  * The file License.txt specifies the terms for use, modification,
  * and redistribution.
  */
@@ -24,7 +24,11 @@
 
 #include <sys/types.h>
 #include <pwd.h>
-#include <md5.h>
+#include <stdarg.h>
+
+#ifdef    SCRAM
+#  include "md5.h"
+#endif /* SCRAM */
 
 #define BINMAIL_IS_SETGID 1
 
@@ -77,19 +81,25 @@
 #  include "scram.h"
 #endif /* SCRAM */
 
+
+#define QPOP_NAME "Qpopper"
+
+
 #define NULLCP          ((char *) 0)
 #define SPACE           32
 #define TAB             9
 #define NEWLINE         '\n'
 
 #define MAXUSERNAMELEN  65
-#define MAXDROPLEN      64
+#define MAXDROPLEN      256
 #define MAXLINELEN      1024
 #define MAXMSGLINELEN   MAXLINELEN
+#define MAXFILENAME     256
 #define MAXCMDLEN       4
 #define MAXPARMCOUNT    5
 #define MAXPARMLEN      10
 #define ALLOC_MSGS      20
+#define OUT_BUF_SIZE    512 /* Amount of output to buffer before forcing a write */
 
 #ifndef MAXHOSTNAMELEN
 #  define MAXHOSTNAMELEN    64
@@ -118,13 +128,6 @@
 #  define SIGPARAM 
 #endif /* SCOR5 or AUX or HPUX */
 
-#ifndef POP_FACILITY
-#  if defined(OSF1) || defined(LINUX)
-#    define POP_FACILITY    LOG_MAIL
-#  else
-#    define POP_FACILITY    LOG_LOCAL0
-#  endif /* OSF1 or Linux */
-#endif /* not defined POP_FACILITY */
 
 typedef enum {
     POP_PRIORITY    =   LOG_NOTICE,
@@ -178,6 +181,17 @@ typedef enum {
 #  define EDQUOT  INT_MIN /* -2147483647-1 on 32-bit systems */
 #endif /* no EDQUOT */
 
+/*
+ * Some OSes don't have the newer errors defined.  If so, define
+ * to be something equivalent
+ */
+#ifndef ECONNABORTED
+#  define ECONNABORTED EPROTO
+#endif /* no ECONNABORTED */
+#ifndef EPROTO
+#  define EPROTO ECONNABORTED
+#endif /* no EPROTO */
+
 /*
  * In standalone mode we need to call _exit rather than exit
  */
@@ -191,26 +205,26 @@ typedef enum {
             /* ^A */
 #define MMDF_SEP_CHAR   '\001'
 
-/* For performance reasons large sites should change the pop tempdrop
- * location to another directory other than the mail spool location.
- * The more files referenced in a directory, the slower the access is
- */
-
 
 #ifndef BULLDIR
 #  define BULLDIR       NULL
 #endif /* BULLDIR */
 
-            /* Defines the maximum number of bulletins a new user
-               receives */
-#ifndef NEWBULLCNT
-#  define NEWBULLCNT    1
-#endif
 
 #ifndef CONTENT_LENGTH
 #  define CONTENT_LENGTH 0
 #endif /* CONTENT_LENGTH */
 
+
+#ifndef MIN
+#  define MIN(A,B) ( ( (A) < (B) ) ? (A) : (B) )
+#endif /* MIN */
+
+#ifndef MAX
+#  define MAX(A,B) ( ( (A) > (B) ) ? (A) : (B) )
+#endif /* MAX */
+
+
 /*
  * Success or failure.  Such a binary world.
  */
@@ -224,6 +238,7 @@ typedef enum {
 
 
 #define POP_TERMINATE   '.'
+#define POP_TERMINATE_S "."
 
 #ifndef POP_TIMEOUT
 #  define POP_TIMEOUT 120 /* timeout connection after this many secs */
@@ -235,6 +250,16 @@ typedef enum {
 
 #define DIG_SIZE    16
 
+typedef struct _pop_tls             pop_tls;            /* defined in pop_tls.h */
+
+/*
+ * We often call pop_write with a literal.  Any half-decent optimizing
+ * compiler can calculate the length of a literal string much easier
+ * and more accurately than I.
+ */
+#define POP_WRITE_LIT(P,STR)    pop_write ( P, STR, strlen(STR) )
+
+
         /* Set these to the types your OS returns if they are not
            already typedefed for you */
 #define OFF_T       off_t
@@ -269,7 +294,7 @@ extern int              errno;
 
 extern int               pop_timeout;
 
-extern volatile int      hangup;
+extern volatile BOOL     hangup;
 
 
 #define pop_command         pop_parm[0]     /*  POP command is first token */
@@ -325,21 +350,95 @@ typedef enum {                               /* Authentication Success  Substate
 } auth_state;
 
 
+/*
+ * Enumeration for chunky (pooled) network writes
+ */
+typedef enum {                               /* Pool network writes:              */
+    ChunkyAlways = 0,                        /* - Always                          */
+    ChunkyTLS,                               /* - Only with TLS/SSL               */
+    ChunkyNever                              /* - Never                           */
+} chunky_type;
+
+
+/*
+ * Enumeration for log-facility option
+ */
+typedef enum {
+    PopLogMail     = LOG_MAIL,
+    PopLogLocal0   = LOG_LOCAL0,
+    PopLogLocal1   = LOG_LOCAL1,
+    PopLogLocal2   = LOG_LOCAL2,
+    PopLogLocal3   = LOG_LOCAL3,
+    PopLogLocal4   = LOG_LOCAL4, 
+    PopLogLocal5   = LOG_LOCAL5,
+    PopLogLocal6   = LOG_LOCAL6,
+    PopLogLocal7   = LOG_LOCAL7
+} log_facility_type;
+
+
+/*
+ * Enumeration for clear text password handling
+ */
 typedef enum {                               /* Clear text passwords are:         */
     ClearTextDefault = 0,                    /* - OK if nothing better available  */
     ClearTextNever,                          /* - never OK                        */
+    ClearTextTLS,                            /* - OK if used with TLS/SSL         */
     ClearTextAlways,                         /* - OK even if APOP password exists */
     ClearTextLocal                           /* - OK on 127.* (local loopback)    */
 } clear_text_type;
 
 
+/*
+ * Enumeration for how TLS/SSL is supported
+ */
+typedef enum {
+    QPOP_TLS_NONE = 0,     /* We don't support TLS/SSL */
+    QPOP_TLS_ALT_PORT,     /* We initiate TLS immediately */
+    QPOP_TLS_STLS          /* We support the STLS command */
+} tls_support_type;
+
+
+/*
+ * Enumeration for TLS/SSL versions
+ */
+typedef enum {
+    QPOP_TLSvDEFAULT = 0,   /* unspecified */
+    QPOP_SSLv2,             /* SSL version 2 only */
+    QPOP_SSLv3,             /* SSL version 3 only */
+    QPOP_TLSv1,             /* TLS version 1 only */
+    QPOP_SSLv23             /* TLSv1, SSLv3, and SSLv2 */
+} tls_vers_type;
+
+
+/*
+ * Enumeration for configuration option types
+ */
 typedef enum {                               /* Config file option is:         */
     CfgBool = 1,                             /* - boolean                      */
     CfgInt,                                  /* - integer                      */
+    CfgMnem,                                 /* - mnemonic                     */
     CfgStr,                                  /* - string                       */
     CfgBad                                   /* - INVALID VALUE                */
 } config_opt_type;
 
+/*
+ * Enumeration for configuration option restrictions
+ */
+typedef enum {                               /* Option restrictions:           */
+    CfgResNone = 1,                          /* - no restrictions              */
+    CfgResUser,                              /* - not valid under user-control */
+    CfgResInit                               /* - not valid after nw traffic   */
+} config_restr_type;
+
+/*
+ * Enumeration for identifying at what stage a config file is being processed
+ */
+typedef enum {                               /* Config file is being processed: */
+    CfgInit =1,                              /* - During our initialization     */
+    CfgConnected,                            /* - After clent traffic           */
+    CfgUser                                  /* - File is under user control    */
+} config_call_type;
+
 
 typedef struct {                                /*  State information for 
                                                     each POP command */
@@ -354,8 +453,6 @@ typedef struct {                                /*  State information for
                                                     the command */
     state       result[2];                      /*  The resulting state after 
                                                     command processing */
-#define success_state   result[0]               /*  State when a command 
-                                                    succeeds (actually when it fails) */
 } state_table;
 
 typedef struct {                                /*  Table of extensions */
@@ -372,7 +469,7 @@ typedef struct {                                /*  Table of extensions */
 typedef struct {                                /*  Table of config file settings */
     char             *name;                     /*  The option name */
     config_opt_type   type;                     /*  The option type */
-    BOOL              restricted;               /*  TRUE if user can't set */
+    config_restr_type restriction;              /*  Restrictions    */
     int               value;                    /*  Value for switch */
 } config_table;
 
@@ -405,24 +502,24 @@ typedef struct {                                /*  Message information */
     BOOL        orig_retr_state;                /*  What was the state at the
                                                     start of this session.
                                                     Used for RSET cmd. */
-    char       *uidl_str;                       /*  Cache of the UIDL str for
+    char        uidl_str [ (DIG_SIZE * 2) + 2 ];/*  Cache of the UIDL str for
                                                     faster access */
 } MsgInfoList;
 
 typedef struct _pop POP;
-#define STACKSIZE 2                             /* Chosen for Implementation */
+#define QPSTACKSIZE 2                           /* Chosen for Implementation */
 typedef void *(*FP)(POP *);
 typedef struct CallStack {
-    FP Stack[STACKSIZE];                        /* Function Pointers array */
+    FP Stack[QPSTACKSIZE];                      /* Function Pointers array */
     int CurP;
 } CALLSTACK;
 
 struct _pop {                                   /*  POP parameter block */
     BOOL                debug;                  /*  Debugging requested */
     BOOL                xmitting;               /*  =1 xtnd xmit started */
-    BOOL                stats;                  /*  Stats requested */
+    BOOL                bStats;                 /*  Stats requested */
     BOOL                dirty;                  /*  Any mailbox changes? */
-    BOOL                kerberos;               /*  Flag to enable kerberos
+    BOOL                bKerberos;              /*  Flag to enable kerberos
                                                     authentication */
     char            *   kerberos_service;       /*  Kerberos service
                                                     being used */
@@ -438,7 +535,8 @@ struct _pop {                                   /*  POP parameter block */
                                                     client IP address */
     unsigned short      ipport;                 /*  Client port for privileged 
                                                     operations */
-    BOOL                downcase_user;          /*  TRUE to downcase user name */
+    BOOL                bDowncase_user;         /*  TRUE to downcase user name */
+    BOOL                bTrim_domain;           /*  TRUE to trim domain from user name */
     char                user[MAXUSERNAMELEN];   /*  Name of the POP user */
     
 #if     defined(__bsdi__) && _BSDI_VERSION >= 199608
@@ -447,8 +545,6 @@ struct _pop {                                   /*  POP parameter block */
     login_cap_t     *   class;                  /*  user's class info */
 #endif /* __bsdi__) && _BSDI_VERSION >= 199608 */
 
-    char                authid[MAXUSERNAMELEN]; /*  Authentication ID (SASL 
-                                                    only) */
     clear_text_type     AllowClearText;         /*  Setting for clear text passwords */
     state               CurrentState;           /*  The current POP operational
                                                     state */
@@ -475,11 +571,22 @@ struct _pop {                                   /*  POP parameter block */
     long                spool_end;              /*  Offset of the end of the
                                                     mailspool */
     FILE            *   drop;                   /*  (Temporary) mail drop */
-    FILE            *   input;                  /*  Input TCP/IP communication 
-                                                    stream */
+    int                 input_fd;               /*  Input TCP/IP communication 
+                                                    file descriptor */
+    char                pcInBuf[2*MAXLINELEN];  /*  Input line buffering */
+    char            *   pcInEnd;
+    char            *   pcInStart;
+    int                 nInBufUsed;             /*  Bytes in input buffer */
+    int                 nInBufStart;            /*  Offset to 1st read byte */
     FILE            *   output;                 /*  Output TCP/IP communication
                                                     stream */
+    char                pcOutBuf[OUT_BUF_SIZE]; /*  Buffer up output so writes
+                                                    to network and TLS layer
+                                                    go in nice big chunks */
+    int                 nOutBufUsed;            /*  Number of bytes in out
+                                                    buffer */
     FILE            *   trace;                  /*  Debugging trace file */
+    char            *   trace_name;             /*  Name of debugging trace file */
     FILE            *   hold;                   /*  In SERVER_MODE, this value
                                                     holds the drop FILE */
     CALLSTACK           InProcess;              /*  Call Stack that holds the
@@ -490,6 +597,7 @@ struct _pop {                                   /*  POP parameter block */
     int                 parm_count;             /*  Number of parameters in 
                                                     parsed list */
     char            *   bulldir;                /*  Bulletin directory */
+    int                 nMaxBulls;              /*  Max bulletins for new users */
     
 #ifdef BULLDB
 #  ifdef GDBM
@@ -501,13 +609,25 @@ struct _pop {                                   /*  POP parameter block */
     int                 bulldb_max_tries;       /*  Max tries to open bulldb */
 #endif
 
-    BOOL                mangle_mime;            /*  turn all MIME into plain
+    BOOL                bNo_mime;               /*  turn all MIME into plain
                                                     text */
     char            *   mmdf_separator;         /*  string between messages */
     char                md5str[BUFSIZ];         /*  String used with the shared
                                                     secret to create the md5
                                                     digest, i.e. APOP server
                                                     challenge */
+    tls_support_type    tls_support;            /* TLS/SSL supported and how */
+    BOOL                tls_started;            /* handshake done */
+    struct _pop_tls *   tls_context;            /* General place to put 
+                                                   TLS context */
+    char            *   tls_identity_file;      /* Single file with all certs, plus
+                                                   private key. */
+    char            *   tls_server_cert_file;   /* File with our certificate. */
+    char            *   tls_private_key_file;   /* File with our private key. */
+    char            *   tls_passphrase;         /* To decrypt private key  */
+    char            *   tls_cipher_list;        /* To restrict ciphers */
+    tls_vers_type       tls_version;            /* To restrict TLS/SSL versions */
+
 #ifdef SCRAM
     char                cchal[ MAXLINELEN ];    /* plain text client challenge */ 
     int                 cchallen;
@@ -515,21 +635,56 @@ struct _pop {                                   /*  POP parameter block */
     int                 schallen;
     SCRAM_MD5_VRFY      scram_verifiers;        /* SCRAM-MD5 verifiers & salt  */
     SCRAM_MD5_INTEGRITY scram_mac;              /* SCRAM-MD5 MAC controls      */
+    char                authid[MAXUSERNAMELEN]; /*  Authentication ID (SASL 
+                                                    only) */
 #endif /* SCRAM */
 
-#ifdef DO_TIMING
     time_t              login_time;             /* Elapsed time to login */
     time_t              init_time;              /* Elapsed time to init drop info */
     time_t              clean_time;             /* Elapsed time to clean up */
-#endif /* DO_TIMING */
 
 #ifdef    DRAC_AUTH
     char            *   drac_host;              /* Host which handles drac */
 #endif /* DRAC_AUTH */
 
-    BOOL                user_opts;              /* process ~/.qpopper-options ? */
+    BOOL                bUser_opts;             /* Process ~/.qpopper-options */
+    BOOL                bSpool_opts;            /* Process .qpopper-options */
+    BOOL                bFast_update;           /* Use rename(2) instead of copying */
+    int                 login_delay;            /* Announced minimum login delay */
+    int                 expire;                 /* Announced message expiration */
+    BOOL                bDo_timing;             /* Generate timing records */
+    BOOL                bCheck_old_spool_loc;   /* Check for temp spool in spool dir despite hashed spool or home spool*/
+    BOOL                bCheck_hash_dir;        /* Check and create hash dirs */
+    BOOL                bCheck_pw_max;          /* Check for expired passwords */
+    BOOL                bUpdate_status_hdrs;    /* Update 'Status:' & 'Message-ID:' */
+    BOOL                bUpdate_on_abort;       /* Go into UPDATE on abort */
+    BOOL                bAuto_delete;           /* Delete msgs */
+    BOOL                bGroup_bulls;           /* Bulletins go to groups */
+    int                 hash_spool;             /* Using hashed spools? */
+    char               *pHome_dir_mail;         /* Home dir is spool loc */
+    BOOL                bOld_style_uid;         /* Generate pre-3.x UIDs */
+    BOOL                bUW_kluge;              /* Ignore UW status msgs */
+    BOOL                bKeep_temp_drop;        /* Keep temp drop file */
+    char               *grp_serv_mode;          /* Group for server mode */
+    char               *grp_no_serv_mode;       /* Group for no server mode */
+    char               *authfile;               /* Authorization file */
+    char               *nonauthfile;            /* Non-authorization file */
+    BOOL                bShy;                   /* Hide version from non-auth users */
+    struct passwd       pw;                     /* Cached info from getpwnam (or elsewhere) */
+    gid_t               orig_group;             /* Original primary group ID */
+    char               *pMail_command;          /* Command to send mail (e.g., path to sendmail) */
+    char               *pCfg_spool_dir;         /* Spool directory */
+    char               *pCfg_temp_dir;          /* Temp spool directory */
+    char               *pCfg_temp_name;         /* Temp spool file name constant */
+    char               *pCfg_cache_dir;         /* Cache directory */
+    char               *pCfg_cache_name;        /* Cache file name constant */
+    chunky_type        chunky_writes;           /* When to pool network writes */
+    BOOL               bNo_atomic_open;         /* open() isn't automic. */
+    log_facility_type   log_facility;           /* Which log facility to use */
+    char               *pLog_login;             /* String to use when logging log-ins */
 };
 
+
 typedef enum { HANGUP,  /* SIGHUP */ 
                TIMEOUT, 
                ABORT    /* Client closes connection */
@@ -574,63 +729,81 @@ typedef enum { HANGUP,  /* SIGHUP */
 
 #endif /* KERBEROS */
 
-#if defined(AUTHFILE)
-   extern int  checkauthfile();
-#endif
 
-#if defined(NONAUTHFILE)
-   extern int  checknonauthfile();
-#endif
+int  pop_apop         (POP *p);
+int  pop_auth         (POP *p);
+int  pop_bull         (POP *p, struct passwd *pwp);
+int  pop_capa         (POP *p);
+int  pop_dele         (POP *p);
+int  pop_dropcopy     (POP *p, struct passwd *pwp);
+int  pop_epop         (POP *p);
+int  pop_euidl        (POP *p);
+int  pop_exit         (POP *p, EXIT_REASON e);
+int  pop_init         (POP *p, int argcount, char **argmessage);
+int  pop_last         (POP *p);
+int  pop_list         (POP *p);
+int  pop_mdef         (POP *p);
+int  pop_parse        (POP *p, char *buf);
+int  pop_pass         (POP *p);
+int  pop_quit         (POP *p);
+int  pop_restore      (POP *p);
+int  pop_rpop         (POP *p);
+int  pop_rset         (POP *p);
+int  pop_scram        (POP *p);
+int  pop_send         (POP *p);
+int  pop_sendline     (POP *p, char *buffer);
+int  pop_stat         (POP *p);
+int  pop_stls         (POP *p);
+struct _pop_tls *pop_tls_init(POP *pPOP);
+int  pop_uidl         (POP *p);
+int  pop_updt         (POP *p);
+int  pop_user         (POP *p);
+void pop_write        (POP *p, char *buffer, int length);
+void pop_write_flush  (POP *p);
+void pop_write_fmt    (POP *p, const char *format, ...) ATTRIB_FMT(2,3);
+void pop_write_line   (POP *p, char *buffer);
+int  pop_xlst         (POP *p);
+int  pop_xmit         (POP *p);
+int  pop_xmit_clean   (POP *p);
+int  pop_xmit_exec    (POP *p);
+int  pop_xmit_recv    (POP *p, char *buffer);
+int  pop_xtnd         (POP *p);
 
-int  pop_apop      (POP *p);
-int  pop_auth      (POP *p);
-int  pop_bull      (POP *p, struct passwd *pwp);
-int  pop_capa      (POP *p);
-int  pop_dele      (POP *p);
-int  pop_dropcopy  (POP *p, struct passwd *pwp);
-int  pop_epop      (POP *p);
-int  pop_euidl     (POP *p);
-int  pop_exit      (POP *p, EXIT_REASON e);
-int  pop_init      (POP *p, int argcount, char **argmessage);
-int  pop_last      (POP *p);
-int  pop_list      (POP *p);
-int  pop_mdef      (POP *p);
-int  pop_parse     (POP *p, char *buf);
-int  pop_pass      (POP *p);
-int  pop_quit      (POP *p);
-int  pop_restore   (POP *p);
-int  pop_rpop      (POP *p);
-int  pop_rset      (POP *p);
-int  pop_scram     (POP *p);
-int  pop_send      (POP *p);
-int  pop_sendline  (POP *p, char *buffer);
-int  pop_stat      (POP *p);
-int  pop_uidl      (POP *p);
-int  pop_updt      (POP *p);
-int  pop_user      (POP *p);
-int  pop_xlst      (POP *p);
-int  pop_xmit      (POP *p);
-int  pop_xmit_clean(POP *p);
-int  pop_xmit_exec (POP *p);
-int  pop_xmit_recv (POP *p, char *buffer);
-int  pop_xtnd      (POP *p);
-
-pop_result pop_config (POP *p, char *config_file, BOOL bUser);
+pop_result pop_config (POP *p, char *config_file, config_call_type CallTime);
+pop_result delete_msg (POP *p, MsgInfoList *mp);
+pop_result undelete_msg(POP *p, MsgInfoList *mp);
+pop_result set_option (POP *p, char *opt_name, char *opt_value);
 
 int  decode64(char *instr, int instrlen, char *outstr, int *outstrlen);
 int  encode64(char *instr, int instrlen, char *outstr, int *outstrlen);
 
-int pop_log(POP *p, log_level  loglev, const char  *fn, size_t ln, const char *format, ... );                  /* parameters for format string */
-int pop_msg(POP *p, pop_result status, const char  *fn, size_t ln, const char *format, ... );
+int pop_log (POP *p, log_level  loglev, const char  *fn, size_t ln, const char *format, ... ) ATTRIB_FMT(5,6);  /* parameters for format string */
+int pop_msg (POP *p, pop_result status, const char  *fn, size_t ln, const char *format, ... ) ATTRIB_FMT(5,6);
+int pop_logv(POP *p, log_level  loglev, const char  *fn, size_t ln, const char *format, va_list ap ) ATTRIB_FMT(5,0); /* parameters for format string */
+
+int  cache_write   (POP *p, int spool_fd, MsgInfoList *mp);
+int  cache_read    (POP *p, int spool_fd, MsgInfoList **mp);
+void cache_unlink  (POP *p);
+
+pop_tls *pop_tls_init(POP *pPOP);
+int      pop_tls_handshake(pop_tls *);
+int      pop_tls_write(pop_tls *, char *, int);
+int      pop_tls_flush(pop_tls *);
+int      pop_tls_read(pop_tls *, char *, int);
+void     pop_tls_shutdown(POP *pPOP);
 
 int  mkstemp       (char *path);
 void pop_lower     (char *buf);
 
-int  check_group(POP *p, struct passwd *pwd, const char *group_name);
-int  isfromline(char *cp);
-void downcase_uname (POP *p, char *q);
-void trim_domain    (POP *p, char *q);
-char *get_state_name ( state curState );
+int   check_group        (POP *p, struct passwd *pwd, const char *group_name);
+int   isfromline         (char *cp);
+void  downcase_uname     (POP *p, char *q);
+void  trim_domain        (POP *p, char *q);
+char *get_state_name     (state curState );
+void  check_config_files (POP *p, struct passwd *pwp );
+int   checkauthfile      (POP *p);
+int   checknonauthfile   (POP *p);
+void  do_log_login       (POP *p);
 
 int  qpopper ( int argc, char *argv[] );
 
@@ -646,7 +819,6 @@ CALLSTACK   *StackInit   (CALLSTACK *);
 FP           GetTop      (CALLSTACK *, int *);
 FP           GetNext     (CALLSTACK *, int *);
 
-char        *mfgets      (char *s, int size, FILE *stream);
 
 char *get_time();
 extern char *ZONE;
@@ -738,5 +910,29 @@ MsgInfoList *msg_ptr ( POP *p, int msg_num );
 #  define DEBUG_LOG9(a,b,a1,a2,a3,a4,a5,a6,a7,a8,a9)
 #endif
 
+#ifdef    _DEBUG
+#  define _DEBUG_LOG0 DEBUG_LOG0
+#  define _DEBUG_LOG1 DEBUG_LOG1
+#  define _DEBUG_LOG2 DEBUG_LOG2
+#  define _DEBUG_LOG3 DEBUG_LOG3
+#  define _DEBUG_LOG4 DEBUG_LOG4
+#  define _DEBUG_LOG5 DEBUG_LOG5
+#  define _DEBUG_LOG6 DEBUG_LOG6
+#  define _DEBUG_LOG7 DEBUG_LOG7
+#  define _DEBUG_LOG8 DEBUG_LOG8
+#  define _DEBUG_LOG9 DEBUG_LOG9
+#else
+#  define _DEBUG_LOG0(a,b)
+#  define _DEBUG_LOG1(a,b,a1)
+#  define _DEBUG_LOG2(a,b,a1,a2)
+#  define _DEBUG_LOG3(a,b,a1,a2,a3)
+#  define _DEBUG_LOG4(a,b,a1,a2,a3,a4)
+#  define _DEBUG_LOG5(a,b,a1,a2,a3,a4,a5)
+#  define _DEBUG_LOG6(a,b,a1,a2,a3,a4,a5,a6)
+#  define _DEBUG_LOG7(a,b,a1,a2,a3,a4,a5,a6,a7)
+#  define _DEBUG_LOG8(a,b,a1,a2,a3,a4,a5,a6,a7,a8)
+#  define _DEBUG_LOG9(a,b,a1,a2,a3,a4,a5,a6,a7,a8,a9)
+#endif /* _DEBUG */
+
 
 #endif /* _POPPER_H */
